;performs the given series of moves on the given CUBE structure
;
;lpCube - ptr to CUBE structure to perform moves on
;lpMoves - ptr to array of one byte move codes
;cbMoves - size of array of cube moves in bytes/moves; this variable is changed during function execution

;send params to it through registers:
;ebx - lpCurrBuf
;edi - lpMoves
;ecx - cbMoves


;PURPOSE
;start with cube state in ESI, use sequence from EDI
;and execute ECX moves on the cube 

;IN
;ESI - ptr to CUBE struct
;EDI - ptr to sequence buffer
;ECX - count of moves to execute in the sequence buffer

;OUT
;the resulting cube is stored back to CUBE struct in ESI

;REMARKS
;


align 16
CubeMove proc uses ebx edi ;lpCube:LPVOID,lpMoves:LPVOID,cbMoves:DWORD

	movc	eax,0
	lea		ebx,[edi+ecx]
align 16
	.repeat
		mov		al,[edi]
		xchg	edx,edx	;filler to avoid stall and to align code
		mov		eax,[eax*sizeof DWORD + offset FuncAddr]
		add		edi,1
		call	eax
		movc	eax,0
	.until edi>=ebx
	ret

	assume esi:ptr CUBE

align 16
Left::
	mov		ah,[esi].CubeSide0.Sqr0
	mov		dh,[esi].CubeSide0.Sqr6
	mov		ch,[esi].CubeSide0.Sqr7
	
	mov		al,[esi].CubeSide4.Sqr0
	mov		dl,[esi].CubeSide4.Sqr6
	mov		cl,[esi].CubeSide4.Sqr7
	mov		[esi].CubeSide0.Sqr0,al
	mov		[esi].CubeSide0.Sqr6,dl
	mov		[esi].CubeSide0.Sqr7,cl
	
	mov		al,[esi].CubeSide2.Sqr2
	mov		dl,[esi].CubeSide2.Sqr3
	mov		cl,[esi].CubeSide2.Sqr4
	mov		[esi].CubeSide4.Sqr0,cl
	mov		[esi].CubeSide4.Sqr6,al
	mov		[esi].CubeSide4.Sqr7,dl
	
	mov		al,[esi].CubeSide5.Sqr0
	mov		dl,[esi].CubeSide5.Sqr6
	mov		cl,[esi].CubeSide5.Sqr7
	mov		[esi].CubeSide2.Sqr2,dl
	mov		[esi].CubeSide2.Sqr3,cl
	mov		[esi].CubeSide2.Sqr4,al
	
	mov		[esi].CubeSide5.Sqr0,ah
	mov		[esi].CubeSide5.Sqr6,dh
	mov		[esi].CubeSide5.Sqr7,ch
	
	mov		eax,dword ptr [esi].CubeSide3
	mov		edx,dword ptr [esi].CubeSide3.Sqr4
	shld	dword ptr [esi].CubeSide3,edx,16
	shld	dword ptr [esi].CubeSide3.Sqr4,eax,16
;	time_end ebx,edi
;	mov		[hInstance+4],eax
	retn

align 16
LeftX::
;00>40	50>00   22>50   40>22
;06>46	56>06   23>56   46>23
;07>47	57>07   24>57   47>24
;3 shr 16
	mov		ah,[esi].CubeSide0.Sqr0
	mov		dh,[esi].CubeSide0.Sqr6
	mov		ch,[esi].CubeSide0.Sqr7
	
	mov		al,[esi].CubeSide5.Sqr0
	mov		dl,[esi].CubeSide5.Sqr6
	mov		cl,[esi].CubeSide5.Sqr7
	mov		[esi].CubeSide0.Sqr0,al
	mov		[esi].CubeSide0.Sqr6,dl
	mov		[esi].CubeSide0.Sqr7,cl
	
	mov		al,[esi].CubeSide2.Sqr2
	mov		dl,[esi].CubeSide2.Sqr3
	mov		cl,[esi].CubeSide2.Sqr4
	mov		[esi].CubeSide5.Sqr0,cl
	mov		[esi].CubeSide5.Sqr6,al
	mov		[esi].CubeSide5.Sqr7,dl
	
	mov		al,[esi].CubeSide4.Sqr0
	mov		dl,[esi].CubeSide4.Sqr6
	mov		cl,[esi].CubeSide4.Sqr7
	mov		[esi].CubeSide2.Sqr2,dl
	mov		[esi].CubeSide2.Sqr3,cl
	mov		[esi].CubeSide2.Sqr4,al
	
	mov		[esi].CubeSide4.Sqr0,ah
	mov		[esi].CubeSide4.Sqr6,dh
	mov		[esi].CubeSide4.Sqr7,ch

	mov		eax,dword ptr [esi].CubeSide3
	mov		edx,dword ptr [esi].CubeSide3.Sqr4
	shrd	dword ptr [esi].CubeSide3,edx,16
	shrd	dword ptr [esi].CubeSide3.Sqr4,eax,16
	retn

align 16
Right::
;02>42	52>02   20>52   42>20
;03>43	53>03   26>53   43>26
;04>44	54>04   27>54   44>27
;1 shl 16
	mov		ah,[esi].CubeSide0.Sqr2
	mov		dh,[esi].CubeSide0.Sqr3
	mov		ch,[esi].CubeSide0.Sqr4
	
	mov		al,[esi].CubeSide5.Sqr2
	mov		dl,[esi].CubeSide5.Sqr3
	mov		cl,[esi].CubeSide5.Sqr4
	mov		[esi].CubeSide0.Sqr2,al
	mov		[esi].CubeSide0.Sqr3,dl
	mov		[esi].CubeSide0.Sqr4,cl
	
	mov		al,[esi].CubeSide2.Sqr0
	mov		dl,[esi].CubeSide2.Sqr6
	mov		cl,[esi].CubeSide2.Sqr7
	mov		[esi].CubeSide5.Sqr2,dl
	mov		[esi].CubeSide5.Sqr3,cl
	mov		[esi].CubeSide5.Sqr4,al
	
	mov		al,[esi].CubeSide4.Sqr2
	mov		dl,[esi].CubeSide4.Sqr3
	mov		cl,[esi].CubeSide4.Sqr4
	mov		[esi].CubeSide2.Sqr0,cl
	mov		[esi].CubeSide2.Sqr6,al
	mov		[esi].CubeSide2.Sqr7,dl
	
	mov		[esi].CubeSide4.Sqr2,ah
	mov		[esi].CubeSide4.Sqr3,dh
	mov		[esi].CubeSide4.Sqr4,ch

	mov		eax,dword ptr [esi].CubeSide1
	mov		edx,dword ptr [esi].CubeSide1.Sqr4
	shld	dword ptr [esi].CubeSide1,edx,16
	shld	dword ptr [esi].CubeSide1.Sqr4,eax,16
	retn

align 16
RightX::
;02>52	42>02   20>42   52>20
;03>53	43>03   26>43   53>26
;04>54	44>04   27>44   54>27
;1 shr 16
	mov		ah,[esi].CubeSide0.Sqr2
	mov		dh,[esi].CubeSide0.Sqr3
	mov		ch,[esi].CubeSide0.Sqr4
	
	mov		al,[esi].CubeSide4.Sqr2
	mov		dl,[esi].CubeSide4.Sqr3
	mov		cl,[esi].CubeSide4.Sqr4
	mov		[esi].CubeSide0.Sqr2,al
	mov		[esi].CubeSide0.Sqr3,dl
	mov		[esi].CubeSide0.Sqr4,cl
	
	mov		al,[esi].CubeSide2.Sqr0
	mov		dl,[esi].CubeSide2.Sqr6
	mov		cl,[esi].CubeSide2.Sqr7
	mov		[esi].CubeSide4.Sqr2,dl
	mov		[esi].CubeSide4.Sqr3,cl
	mov		[esi].CubeSide4.Sqr4,al
	
	mov		al,[esi].CubeSide5.Sqr2
	mov		dl,[esi].CubeSide5.Sqr3
	mov		cl,[esi].CubeSide5.Sqr4
	mov		[esi].CubeSide2.Sqr0,cl
	mov		[esi].CubeSide2.Sqr6,al
	mov		[esi].CubeSide2.Sqr7,dl
	
	mov		[esi].CubeSide5.Sqr2,ah
	mov		[esi].CubeSide5.Sqr3,dh
	mov		[esi].CubeSide5.Sqr4,ch

	mov		eax,dword ptr [esi].CubeSide1
	mov		edx,dword ptr [esi].CubeSide1.Sqr4
	shrd	dword ptr [esi].CubeSide1,edx,16
	shrd	dword ptr [esi].CubeSide1.Sqr4,eax,16
	retn

align 16
Top::
;00>52	42>02   20>42   52>20
;01>53	43>03   26>43   53>26
;02>54	44>04   27>44   54>27
;1 shl 16
	mov		ah,[esi].CubeSide0.Sqr0
	mov		dh,[esi].CubeSide0.Sqr1
	mov		ch,[esi].CubeSide0.Sqr2
	
	mov		al,[esi].CubeSide1.Sqr0
	mov		dl,[esi].CubeSide1.Sqr1
	mov		cl,[esi].CubeSide1.Sqr2
	mov		[esi].CubeSide0.Sqr0,al
	mov		[esi].CubeSide0.Sqr1,dl
	mov		[esi].CubeSide0.Sqr2,cl
	
	mov		al,[esi].CubeSide2.Sqr0
	mov		dl,[esi].CubeSide2.Sqr1
	mov		cl,[esi].CubeSide2.Sqr2
	mov		[esi].CubeSide1.Sqr0,al
	mov		[esi].CubeSide1.Sqr1,dl
	mov		[esi].CubeSide1.Sqr2,cl
	
	mov		al,[esi].CubeSide3.Sqr0
	mov		dl,[esi].CubeSide3.Sqr1
	mov		cl,[esi].CubeSide3.Sqr2
	mov		[esi].CubeSide2.Sqr0,al
	mov		[esi].CubeSide2.Sqr1,dl
	mov		[esi].CubeSide2.Sqr2,cl
	
	mov		[esi].CubeSide3.Sqr0,ah
	mov		[esi].CubeSide3.Sqr1,dh
	mov		[esi].CubeSide3.Sqr2,ch

	mov		eax,dword ptr [esi].CubeSide4.Sqr0
	mov		edx,dword ptr [esi].CubeSide4.Sqr4
	shld	dword ptr [esi].CubeSide4.Sqr0,edx,16
	shld	dword ptr [esi].CubeSide4.Sqr4,eax,16
	retn

align 16
TopX::
	mov		ah,[esi].CubeSide0.Sqr0
	mov		dh,[esi].CubeSide0.Sqr1
	mov		ch,[esi].CubeSide0.Sqr2
	
	mov		al,[esi].CubeSide3.Sqr0
	mov		dl,[esi].CubeSide3.Sqr1
	mov		cl,[esi].CubeSide3.Sqr2
	mov		[esi].CubeSide0.Sqr0,al
	mov		[esi].CubeSide0.Sqr1,dl
	mov		[esi].CubeSide0.Sqr2,cl
	
	mov		al,[esi].CubeSide2.Sqr0
	mov		dl,[esi].CubeSide2.Sqr1
	mov		cl,[esi].CubeSide2.Sqr2
	mov		[esi].CubeSide3.Sqr0,al
	mov		[esi].CubeSide3.Sqr1,dl
	mov		[esi].CubeSide3.Sqr2,cl
	
	mov		al,[esi].CubeSide1.Sqr0
	mov		dl,[esi].CubeSide1.Sqr1
	mov		cl,[esi].CubeSide1.Sqr2
	mov		[esi].CubeSide2.Sqr0,al
	mov		[esi].CubeSide2.Sqr1,dl
	mov		[esi].CubeSide2.Sqr2,cl
	
	mov		[esi].CubeSide1.Sqr0,ah
	mov		[esi].CubeSide1.Sqr1,dh
	mov		[esi].CubeSide1.Sqr2,ch

	mov		eax,dword ptr [esi].CubeSide4.Sqr0
	mov		edx,dword ptr [esi].CubeSide4.Sqr4
	shrd	dword ptr [esi].CubeSide4.Sqr0,edx,16
	shrd	dword ptr [esi].CubeSide4.Sqr4,eax,16
	retn

align 16
Bottom::
	mov		ah,[esi].CubeSide0.Sqr4
	mov		dh,[esi].CubeSide0.Sqr5
	mov		ch,[esi].CubeSide0.Sqr6
	
	mov		al,[esi].CubeSide3.Sqr4
	mov		dl,[esi].CubeSide3.Sqr5
	mov		cl,[esi].CubeSide3.Sqr6
	mov		[esi].CubeSide0.Sqr4,al
	mov		[esi].CubeSide0.Sqr5,dl
	mov		[esi].CubeSide0.Sqr6,cl
	
	mov		al,[esi].CubeSide2.Sqr4
	mov		dl,[esi].CubeSide2.Sqr5
	mov		cl,[esi].CubeSide2.Sqr6
	mov		[esi].CubeSide3.Sqr4,al
	mov		[esi].CubeSide3.Sqr5,dl
	mov		[esi].CubeSide3.Sqr6,cl
	
	mov		al,[esi].CubeSide1.Sqr4
	mov		dl,[esi].CubeSide1.Sqr5
	mov		cl,[esi].CubeSide1.Sqr6
	mov		[esi].CubeSide2.Sqr4,al
	mov		[esi].CubeSide2.Sqr5,dl
	mov		[esi].CubeSide2.Sqr6,cl
	
	mov		[esi].CubeSide1.Sqr4,ah
	mov		[esi].CubeSide1.Sqr5,dh
	mov		[esi].CubeSide1.Sqr6,ch

	mov		eax,dword ptr [esi].CubeSide5.Sqr0
	mov		edx,dword ptr [esi].CubeSide5.Sqr4
	shld	dword ptr [esi].CubeSide5.Sqr0,edx,16
	shld	dword ptr [esi].CubeSide5.Sqr4,eax,16
	retn

align 16
BottomX::
	mov		ah,[esi].CubeSide0.Sqr4
	mov		dh,[esi].CubeSide0.Sqr5
	mov		ch,[esi].CubeSide0.Sqr6
	
	mov		al,[esi].CubeSide1.Sqr4
	mov		dl,[esi].CubeSide1.Sqr5
	mov		cl,[esi].CubeSide1.Sqr6
	mov		[esi].CubeSide0.Sqr4,al
	mov		[esi].CubeSide0.Sqr5,dl
	mov		[esi].CubeSide0.Sqr6,cl
	
	mov		al,[esi].CubeSide2.Sqr4
	mov		dl,[esi].CubeSide2.Sqr5
	mov		cl,[esi].CubeSide2.Sqr6
	mov		[esi].CubeSide1.Sqr4,al
	mov		[esi].CubeSide1.Sqr5,dl
	mov		[esi].CubeSide1.Sqr6,cl
	
	mov		al,[esi].CubeSide3.Sqr4
	mov		dl,[esi].CubeSide3.Sqr5
	mov		cl,[esi].CubeSide3.Sqr6
	mov		[esi].CubeSide2.Sqr4,al
	mov		[esi].CubeSide2.Sqr5,dl
	mov		[esi].CubeSide2.Sqr6,cl
	
	mov		[esi].CubeSide3.Sqr4,ah
	mov		[esi].CubeSide3.Sqr5,dh
	mov		[esi].CubeSide3.Sqr6,ch

	mov		eax,dword ptr [esi].CubeSide5.Sqr0
	mov		edx,dword ptr [esi].CubeSide5.Sqr4
	shrd	dword ptr [esi].CubeSide5.Sqr0,edx,16
	shrd	dword ptr [esi].CubeSide5.Sqr4,eax,16
	retn

align 16
Front::
	mov		eax,dword ptr [esi].CubeSide0.Sqr0
	mov		edx,dword ptr [esi].CubeSide0.Sqr4
	shld	dword ptr [esi].CubeSide0.Sqr0,edx,16
	shld	dword ptr [esi].CubeSide0.Sqr4,eax,16
	
	mov		ah,[esi].CubeSide1.Sqr0
	mov		dh,[esi].CubeSide1.Sqr6
	mov		ch,[esi].CubeSide1.Sqr7
	
	mov		al,[esi].CubeSide4.Sqr4
	mov		dl,[esi].CubeSide4.Sqr5
	mov		cl,[esi].CubeSide4.Sqr6
	mov		[esi].CubeSide1.Sqr0,cl
	mov		[esi].CubeSide1.Sqr6,al
	mov		[esi].CubeSide1.Sqr7,dl
	
	mov		al,[esi].CubeSide3.Sqr2
	mov		dl,[esi].CubeSide3.Sqr3
	mov		cl,[esi].CubeSide3.Sqr4
	mov		[esi].CubeSide4.Sqr4,al
	mov		[esi].CubeSide4.Sqr5,dl
	mov		[esi].CubeSide4.Sqr6,cl
	
	mov		al,[esi].CubeSide5.Sqr0
	mov		dl,[esi].CubeSide5.Sqr1
	mov		cl,[esi].CubeSide5.Sqr2
	mov		[esi].CubeSide3.Sqr2,al
	mov		[esi].CubeSide3.Sqr3,dl
	mov		[esi].CubeSide3.Sqr4,cl
	
	mov		[esi].CubeSide5.Sqr0,dh
	mov		[esi].CubeSide5.Sqr1,ch
	mov		[esi].CubeSide5.Sqr2,ah
	retn

align 16
FrontX::
	mov		eax,dword ptr [esi].CubeSide0.Sqr0
	mov		edx,dword ptr [esi].CubeSide0.Sqr4
	shrd	dword ptr [esi].CubeSide0.Sqr0,edx,16
	shrd	dword ptr [esi].CubeSide0.Sqr4,eax,16
	
	mov		ah,[esi].CubeSide1.Sqr0
	mov		dh,[esi].CubeSide1.Sqr6
	mov		ch,[esi].CubeSide1.Sqr7
	
	mov		al,[esi].CubeSide5.Sqr0
	mov		dl,[esi].CubeSide5.Sqr1
	mov		cl,[esi].CubeSide5.Sqr2
	mov		[esi].CubeSide1.Sqr0,cl
	mov		[esi].CubeSide1.Sqr6,al
	mov		[esi].CubeSide1.Sqr7,dl
	
	mov		al,[esi].CubeSide3.Sqr2
	mov		dl,[esi].CubeSide3.Sqr3
	mov		cl,[esi].CubeSide3.Sqr4
	mov		[esi].CubeSide5.Sqr0,al
	mov		[esi].CubeSide5.Sqr1,dl
	mov		[esi].CubeSide5.Sqr2,cl
	
	mov		al,[esi].CubeSide4.Sqr4
	mov		dl,[esi].CubeSide4.Sqr5
	mov		cl,[esi].CubeSide4.Sqr6
	mov		[esi].CubeSide3.Sqr2,al
	mov		[esi].CubeSide3.Sqr3,dl
	mov		[esi].CubeSide3.Sqr4,cl
	
	mov		[esi].CubeSide4.Sqr4,dh
	mov		[esi].CubeSide4.Sqr5,ch
	mov		[esi].CubeSide4.Sqr6,ah
	retn

align 16
Back::
	mov		ah,[esi].CubeSide1.Sqr2
	mov		dh,[esi].CubeSide1.Sqr3
	mov		ch,[esi].CubeSide1.Sqr4
	
	mov		al,[esi].CubeSide5.Sqr4
	mov		dl,[esi].CubeSide5.Sqr5
	mov		cl,[esi].CubeSide5.Sqr6
	mov		[esi].CubeSide1.Sqr2,al
	mov		[esi].CubeSide1.Sqr3,dl
	mov		[esi].CubeSide1.Sqr4,cl
	
	mov		al,[esi].CubeSide3.Sqr0
	mov		dl,[esi].CubeSide3.Sqr6
	mov		cl,[esi].CubeSide3.Sqr7
	mov		[esi].CubeSide5.Sqr4,dl
	mov		[esi].CubeSide5.Sqr5,cl
	mov		[esi].CubeSide5.Sqr6,al
	
	mov		al,[esi].CubeSide4.Sqr0
	mov		dl,[esi].CubeSide4.Sqr1
	mov		cl,[esi].CubeSide4.Sqr2
	mov		[esi].CubeSide3.Sqr0,cl
	mov		[esi].CubeSide3.Sqr6,al
	mov		[esi].CubeSide3.Sqr7,dl
	
	mov		[esi].CubeSide4.Sqr0,ah
	mov		[esi].CubeSide4.Sqr1,dh
	mov		[esi].CubeSide4.Sqr2,ch
	mov		eax,dword ptr [esi].CubeSide2.Sqr0
	mov		edx,dword ptr [esi].CubeSide2.Sqr4
	shld	dword ptr [esi].CubeSide2.Sqr0,edx,16
	shld	dword ptr [esi].CubeSide2.Sqr4,eax,16
	retn

align 16
BackX::
	mov		ah,[esi].CubeSide1.Sqr2
	mov		dh,[esi].CubeSide1.Sqr3
	mov		ch,[esi].CubeSide1.Sqr4
	
	mov		al,[esi].CubeSide4.Sqr0
	mov		dl,[esi].CubeSide4.Sqr1
	mov		cl,[esi].CubeSide4.Sqr2
	mov		[esi].CubeSide1.Sqr2,al
	mov		[esi].CubeSide1.Sqr3,dl
	mov		[esi].CubeSide1.Sqr4,cl
	
	mov		al,[esi].CubeSide3.Sqr0
	mov		dl,[esi].CubeSide3.Sqr6
	mov		cl,[esi].CubeSide3.Sqr7
	mov		[esi].CubeSide4.Sqr0,dl
	mov		[esi].CubeSide4.Sqr1,cl
	mov		[esi].CubeSide4.Sqr2,al
	
	mov		al,[esi].CubeSide5.Sqr4
	mov		dl,[esi].CubeSide5.Sqr5
	mov		cl,[esi].CubeSide5.Sqr6
	mov		[esi].CubeSide3.Sqr0,cl
	mov		[esi].CubeSide3.Sqr6,al
	mov		[esi].CubeSide3.Sqr7,dl
	
	mov		[esi].CubeSide5.Sqr4,ah
	mov		[esi].CubeSide5.Sqr5,dh
	mov		[esi].CubeSide5.Sqr6,ch
	mov		eax,dword ptr [esi].CubeSide2.Sqr0
	mov		edx,dword ptr [esi].CubeSide2.Sqr4
	shrd	dword ptr [esi].CubeSide2.Sqr0,edx,16
	shrd	dword ptr [esi].CubeSide2.Sqr4,eax,16
	retn

	assume esi:nothing
CubeMove endp

