;prototypes

RCubeWndProc		PROTO	:HWND,:UINT,:WPARAM,:LPARAM
InitCubeCtrl		PROTO	:HINSTANCE
CreateCubeCtrl		PROTO	:HWND,:HINSTANCE,:LPVOID
RCubeCreate			PROTO	:DWORD
RCubePaint			PROTO	:DWORD
RCubeMouseUp		PROTO	:HWND,:LPVOID,:DWORD,:DWORD,:DWORD
RCubeDestroy		PROTO	:DWORD
RCubeContextMenu	PROTO	:HWND,:LPVOID,:DWORD
RCubeMenu			PROTO	:HWND,:DWORD
CalcRubicsCube		PROTO	:LPVOID,:LPVOID
MakeRegions			PROTO	:LPVOID
DestroyRegions		PROTO	:LPVOID
GetCubeColors		PROTO	:LPVOID,:DWORD
SetCubeColors		PROTO	:DWORD,:LPVOID
GetCubeState		PROTO	:DWORD,:DWORD
SetCubeState		PROTO	:DWORD,:DWORD


.const

;CONST for cube control

CUBE_WNDEXTRA	equ 4*3
GWL_HEAP	equ 8
GWL_MEM		equ 4
GWL_LPCUBE	equ 0

SQT_MIDDLE	equ 0
SQT_EDGE	equ 1
SQT_CORNER	equ 2

NOOFFSET	equ -1

;--------------------------cube control messages description
RCM_GETCOLORS	equ WM_USER+500+0
;loads colors of the control's cube sides into the given array of 6 RGBQUAD structures
;wParam - NULL
;lParam - ptr to array of 6 RGBQUAD structures

RCM_SETCOLORS	equ WM_USER+500+1
;stores colors in the given array of 6 RGBQUAD structures into the control's cube sides and redraws
;the control if required
;wParam - BOOL: set to TRUE if the control must be redrawn after colors storing
;lParam - ptr to array of 6 RGBQUAD structures

RCM_GETSTATE	equ WM_USER+500+2
;loads cube state into the given CUBE structure
;wParam - NULL
;lParam - ptr to CUBE structure

RCM_SETSTATE	equ WM_USER+500+3
;stores cube state of the given CUBE structure into the control's cube struct and redraws
;the control if required
;cube state is NOT checked for validity
;wParam - BOOL: set to TRUE if the control must be redrawn after state is stored
;lParam - ptr to CUBE structure

;notification messages
RCN_SIDEMOVE	equ WM_USER+500+0

;-----------------------end of cube control messages description

;default Rubik's Cube coloring
GREEN		equ 0000F000h
RED			equ 000000F0h
CYAN		equ 00F0F000h
BLUE		equ 00FF2020h
WHITE		equ 00FFFFFFh
YELLOW		equ 0000F0F0h

;structures for cubes drawing

RCC_SQRINF struct
	rgbSqr		db ?	;GREEN or RED etc.
	bSqrType	db ?	;SQT_MIDDLE or SQT_EDGE or SQT_CORNER
	currSqr		db ?	;oder of current square in the specified corner cube; can be 0,1 or 2
						;corner squares are counted starting top and going clockwise
						;this value is valid only for SQT_CORNER
	bPad		db ?
	ofsSqr0		dd ?	;NOOFFSET if no valid offset is saved here
	ofsSqr1		dd ?	;NOOFFSET if no valid offset is saved here
RCC_SQRINF ends

RCC_SQR struct
	hRgn		HANDLE ?
	;sqrInfo		RCC_SQRINF<>
	rgbSqr		dd ?	;GREEN or RED etc.
	ptText		POINT <>	;if there is text to be displayed then this is its position
							;it is calculated 
	ptVert		POINT 4 dup (<>)
RCC_SQR ends

RCC_SIDE struct
	Sqr			RCC_SQR 8 dup (<>)
	MidSqr		RCC_SQR<>
RCC_SIDE ends

RCC_CUBE struct
	rgbSide		RGBQUAD 6 dup (<>)
	dwFontHeight	dd ?
	dwPenSize	dd ?
	hCubeletMnu	dd ?
	hMiddleMenu	dd ?
	ptCursPos	POINT <>	;this is used in Context menu processing
	align 16
	lBtnDown	dd ?
	lBtnUp		dd ?
	rBtnDown	dd ?
	rBtnUp		dd ?
	RccSide		RCC_SIDE 6 dup (<>)
RCC_CUBE ends

;macros for cubectrl.asm

;stores ecx to X coord of region with EDI as address base
storeX macro num0,num1,num2,reg
	mov		[edi+num0*sizeof RCC_SIDE].Sqr[num1*sizeof RCC_SQR].ptVert[num2*sizeof POINT].x,reg
endm

storeY macro num0,num1,num2,reg
	mov		[edi+num0*sizeof RCC_SIDE].Sqr[num1*sizeof RCC_SQR].ptVert[num2*sizeof POINT].y,reg
endm

storePoint macro regaddr,num0,num1,num2
	lea		eax,[regaddr].RccSide[num0*sizeof RCC_SIDE].Sqr[num1*sizeof RCC_SQR].ptVert[num2*sizeof POINT]
	push	dword ptr [eax+4]	;.y
	push	dword ptr [eax]		;.x
endm

drawcube macro;uses edi esi
	mov		esi,esp

	storePoint	edi,0,3,1
	storePoint	edi,0,7,2
	storePoint	edi,3,7,3

	storePoint	edi,0,2,1
	storePoint	edi,0,0,2
	storePoint	edi,3,0,3

	storePoint	edi,0,5,2
	storePoint	edi,0,0,0
	storePoint	edi,4,1,3

	storePoint	edi,0,4,2
	storePoint	edi,0,1,0
	storePoint	edi,4,1,0

	storePoint	edi,3,5,2
	storePoint	edi,4,7,2
	storePoint	edi,4,3,1

	storePoint	edi,3,5,3
	storePoint	edi,4,0,2
	storePoint	edi,4,3,0

	storePoint	edi,4,4,1
	storePoint	edi,4,6,2

	storePoint	edi,0,6,2
	storePoint	edi,4,6,2
	storePoint	edi,4,0,3
	storePoint	edi,3,6,3
	storePoint	edi,0,6,2
	storePoint	edi,0,4,1
	storePoint	edi,4,4,1
	storePoint	edi,4,2,0
	storePoint	edi,4,0,3
	mov		eax,esp
	invoke	PolyPolyline,ps.hdc,eax,offset CubeLines,sizeof CubeLines / sizeof DWORD

	storePoint	edi,5,1,2
	storePoint	edi,2,5,2
	storePoint	edi,2,1,1

	storePoint	edi,5,1,3
	storePoint	edi,2,6,2
	storePoint	edi,2,0,1

	storePoint	edi,5,6,2
	storePoint	edi,1,4,2
	storePoint	edi,1,1,0

	storePoint	edi,5,0,1
	storePoint	edi,1,5,2
	storePoint	edi,1,0,0

	storePoint	edi,2,3,2
	storePoint	edi,1,3,1
	storePoint	edi,1,6,3

	storePoint	edi,1,0,2
	storePoint	edi,2,0,3
	storePoint	edi,2,3,1

	storePoint	edi,1,4,1
	storePoint	edi,2,4,2

	storePoint	edi,1,2,0
	storePoint	edi,1,4,1
	storePoint	edi,1,6,2
	storePoint	edi,5,0,2
	storePoint	edi,2,4,2
	storePoint	edi,2,2,1
	storePoint	edi,1,2,0
	storePoint	edi,1,0,3
	storePoint	edi,1,6,2
	mov		eax,esp
	invoke	PolyPolyline,ps.hdc,eax,offset CubeLines,sizeof CubeLines / sizeof DWORD
	mov		esp,esi
endm

;the following structures are defined here but uses with the application itself
;side structure
SIDE struct
    Sqr0 db ?
    Sqr1 db ?
    Sqr2 db ?
    Sqr3 db ?
    Sqr4 db ?
    Sqr5 db ?
    Sqr6 db ?
    Sqr7 db ?
SIDE ends

;cube structure
CUBE struct
    CubeSide0 SIDE<>
    CubeSide1 SIDE<>
    CubeSide2 SIDE<>
    CubeSide3 SIDE<>
    CubeSide4 SIDE<>
    CubeSide5 SIDE<>
CUBE ends

sqr_offs macro num0,num1
	exitm<num0*sizeof RCC_SIDE+num1*sizeof RCC_SQR>
endm


align 16
;this data is const data for cubectrl.inc
;const data with info for all squares of the cube
;with their colors, types (corner,edge,middle) and their relation with other squares
SqrInfo	RCC_SQRINF  <0,SQT_CORNER,1,0,sqr_offs(4,6),sqr_offs(3,2)>,\
					<0,SQT_EDGE  ,0,0,sqr_offs(4,5),NOOFFSET>,\
					<0,SQT_CORNER,2,0,sqr_offs(4,4),sqr_offs(1,0)>,\
					<0,SQT_EDGE  ,0,0,sqr_offs(1,7),NOOFFSET>,\
					<0,SQT_CORNER,2,0,sqr_offs(1,6),sqr_offs(5,2)>,\
					<0,SQT_EDGE  ,0,0,sqr_offs(5,1),NOOFFSET>,\
					<0,SQT_CORNER,0,0,sqr_offs(5,0),sqr_offs(3,4)>,\
					<0,SQT_EDGE  ,0,0,sqr_offs(3,3),NOOFFSET>,\
					<0,SQT_MIDDLE,0,0,NOOFFSET,NOOFFSET>
SqrInf1 RCC_SQRINF  <1,SQT_CORNER,1,0,sqr_offs(4,4),sqr_offs(0,2)>,\
					<1,SQT_EDGE  ,0,0,sqr_offs(4,3),NOOFFSET>,\
					<1,SQT_CORNER,2,0,sqr_offs(4,2),sqr_offs(2,0)>,\
					<1,SQT_EDGE  ,0,0,sqr_offs(2,7),NOOFFSET>,\
					<1,SQT_CORNER,2,0,sqr_offs(2,6),sqr_offs(5,4)>,\
					<1,SQT_EDGE  ,0,0,sqr_offs(5,3),NOOFFSET>,\
					<1,SQT_CORNER,0,0,sqr_offs(5,2),sqr_offs(0,4)>,\
					<1,SQT_EDGE  ,0,0,sqr_offs(0,3),NOOFFSET>,\
					<1,SQT_MIDDLE,0,0,NOOFFSET,NOOFFSET>
SqrInf2	RCC_SQRINF  <2,SQT_CORNER,1,0,sqr_offs(4,2),sqr_offs(1,2)>,\
					<2,SQT_EDGE  ,0,0,sqr_offs(4,1),NOOFFSET>,\
					<2,SQT_CORNER,2,0,sqr_offs(4,0),sqr_offs(3,0)>,\
					<2,SQT_EDGE  ,0,0,sqr_offs(3,7),NOOFFSET>,\
					<2,SQT_CORNER,2,0,sqr_offs(3,6),sqr_offs(5,6)>,\
					<2,SQT_EDGE  ,0,0,sqr_offs(5,5),NOOFFSET>,\
					<2,SQT_CORNER,0,0,sqr_offs(5,4),sqr_offs(1,4)>,\
					<2,SQT_EDGE  ,0,0,sqr_offs(1,3),NOOFFSET>,\
					<2,SQT_MIDDLE,0,0,NOOFFSET,NOOFFSET>
SqrInf3	RCC_SQRINF  <3,SQT_CORNER,1,0,sqr_offs(4,0),sqr_offs(2,2)>,\
					<3,SQT_EDGE  ,0,0,sqr_offs(4,7),NOOFFSET>,\
					<3,SQT_CORNER,2,0,sqr_offs(4,6),sqr_offs(0,0)>,\
					<3,SQT_EDGE  ,0,0,sqr_offs(0,7),NOOFFSET>,\
					<3,SQT_CORNER,2,0,sqr_offs(0,6),sqr_offs(5,0)>,\
					<3,SQT_EDGE  ,0,0,sqr_offs(5,7),NOOFFSET>,\
					<3,SQT_CORNER,0,0,sqr_offs(5,6),sqr_offs(2,4)>,\
					<3,SQT_EDGE  ,0,0,sqr_offs(2,3),NOOFFSET>,\
					<3,SQT_MIDDLE,0,0,NOOFFSET,NOOFFSET>
SqrInf4	RCC_SQRINF  <4,SQT_CORNER,0,0,sqr_offs(3,0),sqr_offs(2,2)>,\
					<4,SQT_EDGE  ,0,0,sqr_offs(2,1),NOOFFSET>,\
					<4,SQT_CORNER,0,0,sqr_offs(2,0),sqr_offs(1,2)>,\
					<4,SQT_EDGE  ,0,0,sqr_offs(1,1),NOOFFSET>,\
					<4,SQT_CORNER,0,0,sqr_offs(1,0),sqr_offs(0,2)>,\
					<4,SQT_EDGE  ,0,0,sqr_offs(0,1),NOOFFSET>,\
					<4,SQT_CORNER,0,0,sqr_offs(0,0),sqr_offs(3,2)>,\
					<4,SQT_EDGE  ,0,0,sqr_offs(3,1),NOOFFSET>,\
					<4,SQT_MIDDLE,0,0,NOOFFSET,NOOFFSET>
SqrInf5	RCC_SQRINF  <5,SQT_CORNER,1,0,sqr_offs(0,6),sqr_offs(3,4)>,\
					<5,SQT_EDGE  ,0,0,sqr_offs(0,5),NOOFFSET>,\
					<5,SQT_CORNER,1,0,sqr_offs(1,6),sqr_offs(0,4)>,\
					<5,SQT_EDGE  ,0,0,sqr_offs(1,5),NOOFFSET>,\
					<5,SQT_CORNER,1,0,sqr_offs(2,6),sqr_offs(1,4)>,\
					<5,SQT_EDGE  ,0,0,sqr_offs(2,5),NOOFFSET>,\
					<5,SQT_CORNER,1,0,sqr_offs(3,6),sqr_offs(2,4)>,\
					<5,SQT_EDGE  ,0,0,sqr_offs(3,5),NOOFFSET>,\
					<5,SQT_MIDDLE,0,0,NOOFFSET,NOOFFSET>
DefColors	dd GREEN,RED,CYAN,BLUE,WHITE,YELLOW
CubeLines	dd 9,2,3,3,3,3,3,3

IDM_ROTCBCW	equ 3000
IDM_ROTCBCCW	equ 3001
IDM_MOVCB	equ 3002
IDM_ROTCW	equ 3003
IDM_ROTCCW	equ 3004
IDM_SIDCLR	equ 3005
IDM_ADDR	equ 3006

align 4
msRotCbCW	db "Rotate Cubelet CW",9,"Ctrl+Right Click",0
msRotCbCCW	db "Rotate Cubelet CCW",9,"Ctrl+Left Click",0
msMoveCb	db "Move Cubelet",9,"Ctrl+Drag",0
msRotCW		db "Rotate Side CW",9,"Drag CW"
msEmpty		db 0
msRotCCW	db "Rotate Side CCW",9,"Drag CCW",0
msSideCol	db "Set Side Color...",0
msAddr		db "Side %d: %s (%c); Square %d",0
msSideName	db 'Side Name: "%s" (%c)',0
align 4
cSideChars	db "F","R","B","L","T","U",0,0	;chars for sides
szSideNames	db "Front",0,"Right",0,"Back",0,"Left",0,"Top",0,"Underside",0
szArial		db "Arial",0
align 4

