;--------------------------------------------------------------------------------------

;this is to be a big function
;it will search for a sequence of moves that will change lpSrcCube into lpDestCube
;use the given buffer (lpBufs) as a structure for search

;RET
;NULL if not found or destination cube is equal to source cube
;return amount of moves to reach dest cube, the required sequence or moves
;will be stored in FINDSEQ.SeqBufferOpt array

;REMARKS
;found sequence will be returned in FINDSEQ.SeqBufferOpt array
;length of the array will be returned with the function

;	mov		nResult,100000000
;	.repeat
;		call	LeftXmm
;		dec		nResult
;	.until ZERO?
;	jmp		@exit
	;LeftXmm	828ms
	;LeftXmmX	828ms
	;RightXmm	1.000-1.031ms
	;RightXmmX	953ms
	;TopXmm		1.047ms
	;TopXmmX	1.047ms
	;BottomXmm	1.047ms
	;BottomXmmX	1.047ms
	;FrontXmm	1.344ms
	;FrontXmmX	1.344ms
	;BackXmm	1.328-1.344ms
	;BackXmmX	1.344ms


align 16
FindSeq proc uses ebx esi edi lpDestCube:LPVOID,lpSrcCube:LPVOID,lpSeq:LPVOID,nMaxMoves:DWORD,boolFast:DWORD
	LOCAL	lpCubeMove:LPVOID
	LOCAL	nResult:DWORD

	movc	nResult,FALSE
	invoke	GetTickCount
	mov		dwStartTime,eax
	mov		edx,MAX_MOVES	;this is the max amount of moves that can be contained in the buffer
	mov		eax,nMaxMoves
	cld
	.if eax	;if nMaxMoves>0 then start search
		.if edx<eax		;edx<nMaxMoves
		;make sure that nMaxMoves is not bigger than the length of moves array
			mov		nMaxMoves,edx
		.endif
		mov		ebx,offset FindSequence
		;FindSequence is a special structure (FINDSEQ) that contains
		;source cube state, dest. cube state, processed sequence
		assume ebx:ptr FINDSEQ
		;prepare the buffer
		;copy source cube from lpSrcCube to FindSequence.SrcCube
		mov		esi,lpSrcCube
		mov		ecx,sizeof CUBE / sizeof DWORD
		lea		edi,[ebx].SrcCube
		push	ecx
			rep		movsd
		pop		ecx			;invoke	RtlMoveMemory,addr [ebx].SrcCube,lpSrcCube,sizeof CUBE
		;copy dest cube from lpDestCube to FindSequence.DestCube
		mov		esi,lpDestCube
		lea		edi,[ebx].DestCube
		rep		movsd		;invoke	RtlMoveMemory,addr [ebx].DestCube,lpDestCube,sizeof CUBE
		;if SSE2 is supported then prepare constants in XMM6, XMM7 registers
		;below we also store ptr to the respective CmpCube function
		;@CmpCube takes the source cube, performs specified moves on it and compares it to the dest cube.
		;@CmpCubeXmm performs specified moves on the source cube and compares it ot the dest cube.
		;In latter one all these things are done using XMM registers and SSE,SSE2 insturctions.
		.if boolFast
			xmm_setconst	;macro that prepares constants in XMM registers
			mov		esi,offset @CmpCubeXmm
		.else
			mov		esi,offset @CmpCube
		.endif
		mov		edi,offset OneMoves	;array of one-move sequences
		mov		lpCubeMove,esi		;store ptr to the proper functions into lpCubeMove
		;first of all we will go through all of the 1-move sequences.
		;we will use OneMoves array to go through the sequences.
		;however this loop is not very critical.
		.repeat
			mov		ecx,1
			;EDI is always ready and it is pointing to the array of sequences
			;EBX - ptr to FINDSEQ struct aligned by 16 bytes boundary
			;ECX - count of moves to perform
			call	esi		;invoke	CubeMove,ebx,edi,1
			;upon return from @CmpCube the CARRY flag is set if the sequence is found
			jc		@found1	;.break .if CARRY?	;one-move sequence was found
			add		edi,1	;inc ptr to point at the next sequence
			mov		esi,lpCubeMove	;ESI = ptr to @CmpCube or @CmpCubeXmm
		.until edi>=offset OneMoves+sizeof OneMoves	;until end of sequence array found
		;.if !CARRY?;!eax
			cmp		nMaxMoves,2	;max sequence is 1 then exit
			jb		@exit
			;now we need to go through all 2-move sequences
			;we will use TwoMoves array for this purpose
			mov		edi,offset TwoMoves
			;this loop is not very critical either
			.repeat
				mov		ecx,2
				call	esi	;invoke	CubeMove,ebx,edi,2
				jc		@found2;.break.if CARRY?	;eax
				add		edi,2
				mov		esi,lpCubeMove
			.until edi>=offset TwoMoves + CNT_TWO_MOVES * 2
			;.if !CARRY?;!eax
				cmp		nMaxMoves,3	;if max limit is 2 then exit (1 was checked before)
				jb		@exit
				;fill FindSequence.SeqBuffer with -1
				;this code also ensures that entire .SeqBuffer is in cache
				lea		edi,[ebx].SeqBuffer
				movc	eax,-1
				mov		ecx,(sizeof FINDSEQ.SeqBuffer + sizeof FINDSEQ.dwPad) / sizeof DWORD
				push	edi	;preserve ptr to .SeqBuffer for later use
					rep		stosd
				pop		edi
				;invoke	RtlFillMemory,edi,sizeof FINDSEQ.SeqBuffer + sizeof FINDSEQ.dwPad,-1
				mov		nCurrMoves,3	;we will start with 3-move sequences
				;Now we need to begin search starting with 3-move sequences. The first possible sequence is
				;CM_LEFT,CM_LEFT,CM_LEFT; but it can be optimized into CM_LEFTX which is a 1-move
				;sequence and we have prevoiusly tried all 1-move sequences. Next is
				;CM_LEFTX,CM_LEFT,CM_LEFT; but CM_LEFTX and CM_LEFT are opposite moves so they will
				;be eliminated by optimizer and we will have 1 CM_LEFT move left. We have tried all
				;1-move sequences. Next is
				;CM_RIGHT,CM_LEFT,CM_LEFT. This one will not be changed by optimizer. So we can use
				;it as a starting sequence.
				;However NextSeq() makes one step ahead, so if we want to start
				;with CM_RIGHT,CM_LEFT,CM_LEFT then we need to put CM_LEFTX,CM_LEFT,CM_LEFT into
				;sequence buffer, NextSeq() will step it to CM_RIGHT,CM_LEFT,CM_LEFT and we will
				;actually start with this initial sequence.
				mov		dword ptr [edi],CM_LEFTX + (CM_LEFT shl 8) + (CM_LEFT shl 16) + (-1 shl 24)
				;In the following loop we need to do this:
				;1. Advance move sequence in .SeqBuffer to next sequence. This used to
				;	be NextSeq() function.
				;2. Check if any portion of current sequence can be eliminated or changed to make a smaller sequence.
				;	If current sequence can be shrinked even by one move then it becomes one of the sequences that
				;	we have tried already, so we do not need to try it again, we skip it and go to advance
				;	to the next sequence. Then do checking again until an unshrinkable sequence is found.
				;	When final unshrinkable sequence is obtained, then execute the sequence and check
				;	if it can do the desired transition. So compare the result with the dest cube state
				;must enter the loop with EAX==0,ECX==nCurrMoves,EDX==nMaxMoves
				mov		ecx,nCurrMoves
				mov		esi,lpCubeMove
				movc	eax,0
align 16
				.repeat
				;here EAX must always be 0
					inc		eax		;eax=1	;mov		eax,1
					movc	edx,0	;movc	esi,0	;esi=0
@nextseq:
					add		eax,[edi+edx]	;next move = 1 + prev move
					mov		edx,edx	;filler to avoid stall
					cmp		al,CM_BACKX	;.if al<=CM_BACKX
					;if curr move is bigger than max possibel move (CM_BACKX) then advance to next move within seq
					ja		@nextmove
					mov		[edi+edx],eax	;save successfully incremented move to update the sequence
					;now check if this sequence can be shrinked
					;here we check for two cases:
					;1. opposite moves next to each other
					;2. at least 3 same moves in a row
align 16
					.repeat
						mov		eax,[edi+edx]	;get four closest chars from sequence
						lea		edx,[edx]		;filler for code alignment and stall prevention
						xor		al,ah		;xor move0 and move1
						jz		@0eq1	;if ZERO then 1st two chars are equal
										;so go to check the third char to be equal
						cmp		al,1	;if ONE then these were opposite moves
						je		@contra	;go to skip this sequence and advance to the next possible sequence
@retloop:
						dec		edx
					.until SIGN?
					;this test allows us to stop the search by changing the flag in glogal valiable
					test	bStats,fSEARCHING
					jz		@out	;jump to jmp @exit ;if go straight to @exit then we get a long version of jump (6 bytes)
					;we need to restore nCurrMoves here since it is changed in CubeMove but not in CubeMoveXmm
					call	esi;lpCubeMove	;call	CubeMove,ebx,edi,ecx
				.until CARRY?;eax
				;go to @found if current sequence can do the desired transition
				jmp		@found
align 8
;now we know that move0 == move1; so here we check if move1 == move2
@0eq1:	;.if ah==byte ptr [edi+esi+2];if 3 same moves in a row
						shr		eax,8	;shift EAX right to put move1 and move2 into AL and AH
						cmp		al,ah	;cmp		ah,byte ptr [edi+esi+2]
						jne		@retloop;if move1 not equal to move2 then return to inner loop
						;else EAX=1 and go to find next sequance
align 8
;now move0 was opposite of move1; so put 1 in EAX and go to check the next loop
@contra:
						mov		eax,1	;if opposite moves were found then go to check the next sequence
						jmp		@nextseq
align 8
@nextmove:
					;we get here if there was an "overflow" with moves increment
					;so put CM_LEFT (null) into current move and go to inc the next move
					movc	byte ptr [edx+edi],CM_LEFT
					inc		edx
					mov		eax,1
					;if incrementing we have not reached the highest num yet
					;then just continue without incrementing current length of moves in sequence buffer
					;.continue .if esi<ecx
					cmp		edx,ecx		;cmp	esi,nCurrMoves
					jb		@nextseq
					;---------below this line we have randomly executed code 
					;we get here only when extending current sequence search length
					;at this point EAX must be 1 always
					add		ecx,eax	;ecx+1
					inc		nCurrMoves
					cmp		ecx,nMaxMoves	;if curr pos in moves buffer reaches buffer end then exit
					jbe		@nextseq
@out:	;we use this extra jump to make the loop smaller (reduce jump size after test bStats,fSEARCHING)
					jmp		@exit
;				;if it has never been found then leave 0 in nResult and exit
@found:
				mov		ecx,nCurrMoves
				push	ecx
					mov		esi,lpSeq
					invoke	RtlMoveMemory,esi,edi,ecx
				pop		ecx
				movc	byte ptr [esi+ecx],-1
				jmp		@result
			;.endif
@found2:
			movc	eax,-1
			mov		edx,lpSeq
			mov		ax,word ptr [edi]
			movc	ecx,0
			mov		dword ptr [edx],eax
			mov		cl,2
			jmp		@result
		;.else
@found1:
			movc	eax,-1
			mov		edx,lpSeq
			mov		al,byte ptr [edi]
			movc	ecx,1
			mov		dword ptr [edx],eax
		;.endif
@result:
		mov		nResult,ecx
@exit:
	.endif
	return nResult

;EBX - ptr FINDSEQ struct
;ESI - ptr to this function
;EDI - ptr to sequence buffer
;ECX - amount of moves to execute from the buffer
align 16
@CmpCubeXmm:
	movdqa	xmm0,[ebx].SrcCube.CubeSide0
	movdqa	xmm1,[ebx].SrcCube.CubeSide2
	movdqa	xmm2,[ebx].SrcCube.CubeSide4
	movc	eax,0
	sub		edx,edx	;use EDX as offset into lpMoves in EDI
align 16
	.repeat
		mov		al,[edi+edx]
		xchg	ecx,ecx	;filler to avoid stall and to align code
		mov		eax,[eax*sizeof DWORD + offset XmmFuncAddr]
		add		edx,1;inc		edx
		call	eax
		movc	eax,0
	.until edx>=ecx
	;we don't use call to CubeMoveXmm to speed up execution
	;instead we inserted that same code in here
	;now compare cubes with SSE2 too
		;the following system (with pxor instead of pcmpeqb) runs faster
	pxor	xmm0,[ebx].DestCube.CubeSide0
	pxor	xmm1,[ebx].DestCube.CubeSide2
	pxor	xmm2,[ebx].DestCube.CubeSide4
	por		xmm0,xmm1
	por		xmm0,xmm2
	packsswb	xmm0,xmm0
	packsswb	xmm0,xmm0
	movd		edx,xmm0
	movc	eax,0
	sub		edx,1
	retn
	
;	pcmpeqb	xmm0,[ebx].DestCube.CubeSide0
;	pcmpeqb	xmm1,[ebx].DestCube.CubeSide2
;	pcmpeqb	xmm2,[ebx].DestCube.CubeSide4
;	pand	xmm0,xmm1
;	pand	xmm0,xmm2
;	;if ax==-1 then cubes match so return TRUE
;	;else cubes don't match so return FALSE
;	pmovmskb	edx,xmm0
;	;movc	eax,0
;	add		dx,1
;	;adc		eax,eax
;	retn

;EBX - ptr FINDSEQ struct
;ESI - ptr to this function
;EDI - ptr to sequence buffer
;ECX - amount of moves to execute from the buffer
;ECX must be preserved!
align 16
@CmpCube:
	push	ecx
	push	esi
	push	edi
		mov		edx,ecx
			xchg	eax,edi
				lea		esi,[ebx].SrcCube
				lea		edi,[ebx].CurrCube
				mov		ecx,sizeof CUBE / sizeof DWORD
				push	edi
					rep		movsd
				pop		esi
			xchg	eax,edi
		mov		ecx,edx
		call	CubeMove
		lea		edi,[ebx].DestCube
		;now ESI is still pointing to CurrCube where we have the result of moves
		;and EDI is pointing to DestCube with which we will compare resulting cube in CurrCube
		mov		ecx,sizeof CUBE / sizeof DWORD
		movc	eax,0
		repe	cmpsd
		setne	al
	pop		edi
	pop		esi
	pop		ecx
	sub		eax,1
	mov		eax,0	;make sure we output NULL in EAX
	retn

;	;mov		esi,offset dbSide;lpCube	;offset dbSide;
;	;mov		edi,lpMoves
;	;mov		ecx,cbMoves
;	assume ebx:ptr FINDSEQ
;	lea		edx,[ebx].DestCube
;	;from now on EDX may not be changed
;		mov		esi,ebx	;lea		esi,[ebx].SrcCube
;		assume esi:ptr CUBE
;		;ESI is always lpCube
;		;EDI is lpMoves
;		movdqa	xmm0,[esi].CubeSide0
;		movc	eax,0
;		xmm_mov	xmm1,[esi].CubeSide2
;		lea		ebx,[edi+ecx]	;calc lpEnd for the latter loop
;		xmm_mov	xmm2,[esi].CubeSide4
;		;assume is done for the functions themselves
;align 16
;		.repeat
;			mov		al,[edi]
;			mov		eax,[eax*sizeof DWORD + offset XmmFuncAddr]
;			add		edi,1
;			call	eax
;			movc	eax,0
;		.until edi>=ebx
;		;save data to memory
;;		movdqa	[esi].CubeSide0,xmm0
;;		movdqa	[esi].CubeSide2,xmm1
;;		movdqa	[esi].CubeSide4,xmm2
;	mov		edi,edx
;	assume edi:ptr CUBE
;	;compare cubes with SSE2 too
;	pcmpeqb	xmm0,[edi].CubeSide0
;	pcmpeqb	xmm1,[edi].CubeSide2
;	pcmpeqb	xmm2,[edi].CubeSide4
;	pand	xmm0,xmm1
;	movc	eax,FALSE
;	pand	xmm0,xmm2
;	;if ax==-1 then cubes match so return TRUE
;	;else cubes don't match so return FALSE
;	pmovmskb	edx,xmm0
;	add		dx,1
;	adc		eax,eax
;	ret


;optimization loop
;							.repeat
;								mov		al,[edi+esi]
;								mov		ah,[edi+esi+1]	;next char
;								xor		al,ah
;								.if !ZERO?
;									;.if al==1
;									cmp		al,1
;									jne		@nxch
;									mov		ah,0
;									jmp		@nextseq
;								.endif
;								.if ah==byte ptr [edi+esi+2];if 3 same moves
;									mov		eax,1
;									jmp		@nextseq
;								.endif
;@nxch:
;								sub		esi,1
;							.until SIGN?
;							.break

assume ebx:nothing

FindSeq endp

