;SEH struct

SEH_FRAME struct
    lpPrevSEHStruct dd ?
    lpCurrSEH       dd ?
    lpSafePlace     dd ?
    regESP          dd ?
    regEBP          dd ?
SEH_FRAME ends

;MACROS

GetAttr macro arg:REQ,attr
    ifnb <attr>
        exitm <attr>
    else
        exitm <OPATTR(arg)>
    endif
endm

;this macro checks if given argument is a register
IsReg macro arg,atr
    attr = GetAttr(arg,atr)
    if attr eq 30h
        exitm <attr>
    else
        exitm <0>
    endif
endm

;this macro checks if given argument is a constant
IsConst macro arg,atr
    attr = GetAttr(arg,atr)
    if attr eq 24h
        exitm <attr>
    else
        exitm <0>
    endif
endm

;this macro checks if given argument is a global var
IsGlobal macro arg,atr
    attr = GetAttr(arg,atr)
    if attr eq 2Ah
        exitm <attr>
    else
        exitm <0>
    endif
endm

;this macro checks if given argument is a local var ([ebp+X] or [esp+X])
IsLocal macro arg,atr
    attr = GetAttr(arg,atr)
    if attr EQ 62h
        exitm <attr>
    else
        exitm <0>
    endif
endm

;
IsRelVar macro arg,atr
    attr = GetAttr(arg,atr)
    if (attr EQ 22h) OR (attr EQ 62h)
        exitm <attr>
    else
        exitm <0>
    endif
endm

IsCodeRef macro arg,atr
    attr = IsFunc(arg,atr)
    ife attr
        attr = IsLabel(arg,atr)
    endif
    exitm <attr>
endm

IsLabel macro arg,atr
    attr = GetAttr(arg,atr)
    if attr EQ 25h
        exitm <attr>
    else
        exitm <0>
    endif
endm

;this macro is used to find out if the "arg" is a function reference
IsFunc macro arg,atr
    attr = GetAttr(arg,atr)
    ;(attr EQ 3A5h) OR (attr EQ 1A5h) OR (attr EQ 3AAh) OR (attr EQ 1AAh)
    ;if it's a STDCALL func or C func or var to STDCALL func or var to C func
    attrhi = attr shr 8
    attrlo = attr and 00FFh
    if attrhi
    ;if it's a function reference then attrhi will contain language type of function
    ;else it's a code label
    ;here is the list of language types:
    ;0 - no language type
    ;1 - C
    ;2 - SYSCALL
    ;3 - STDCALL
    ;4 - Pascal
    ;5 - FORTRAN
    ;6 - Basic
        if (attrlo EQ 0A5h) OR (attrlo EQ 0AAh) OR (attrlo EQ 25h) OR (attrlo EQ 2Ah)
        ;if A5h then it is a constant value and it's an address of a function
        ;if AAh then it is a variable that points to an address of a function
        ;if 25h then it is a constant address of a function that has no PROTO declaration
        ;if 2Ah then it is a variable that points to a function without a PROTO declaration
            exitm <attr>
        else
            exitm <0>
        endif
    else
        exitm <0>
    endif
endm

;This macro is designed to find out the size of the register in bits (8,16,32)
;So if arg is a valid register then a valid register size will be returned
;else zero will be returned indicating that arg was not a register at all
@RegSize macro arg
	if IsReg(arg)
		exitm <(TYPE arg) shl 3>	;TYPE arg * 8
	else
		exitm <0>
	endif
endm


movc macro var:REQ,const:REQ,reg    ;move constant into a var or a reg
;this macro moves a constant into a given var or register
    regsize equ @RegSize(var)
    regregsize equ 0
    ifdef reg
    	regregsize equ @RegSize(reg)
    endif
    if (regsize EQ 32 OR regsize EQ 16 OR regsize EQ 8)
    ;if reg then var is a register so do accordingly
        if const eq 0
            xor     var,var
        elseif const eq 1
            xor     var,var
            inc     var
        elseif const eq -1
            or      var,-1
        elseif const eq -2
            or      var,-1
            dec     var
        else
            mov     var,const
        endif
;    elseif (regsize EQ 80)    ;if FPU
;        fsub    var,var
;    elseif (regsize EQ 64)
;        xor     var,var ;just to bring me here if I use MMX
;    elseif (regsize EQ 128)
;        xor     var,var
    else
    ;else var is a variable
        if regregsize NE 0 
            ife const
                xor     reg,reg
            elseif const eq -1
                or      reg,-1
            else
                mov     reg,const
            endif
            mov     var,reg
        else
            ife const
                and     var,const
            elseif const eq -1
                or      var,const
            else
                mov     var,const
            endif
        endif
    endif
endm

alignValue macro arg1,argAlign
    and     arg1,-argAlign
    add     arg1,argAlign
endm

alignValueMin macro arg1,argAlign
    .if arg1 & argAlign-1
        alignValue arg1,argAlign
    .endif
endm

clrfl macro var,flg
    and     var,not (flg)
endm

setfl macro var,flg
    or      var,flg
endm

notfl macro var,flg
    xor     var,flg
endm

minusOne macro var
    movc    var,-1
endm

zeroReg macro reg
    xor     reg,reg ;movc    reg,0
endm

zeroVar macro var
    movc    var,NULL
endm

regTrue macro reg
    movc    reg,TRUE
endm

makelong macro val1,val2
	exitm<val1 + (val2 shl 16)>
endm

;loads 64bit value to registers
lod64b macro var64,regLo,regHi
    mov     regLo,dword ptr [var64]
    mov     regHi,dword ptr [var64 + sizeof DWORD]
endm

sto64b macro var64,regLo,regHi
    mov     dword ptr [var64],regLo
    mov     dword ptr [var64 + sizeof DWORD],regHi
endm

add64b2Mem macro var64,regLo,regHi
    add     dword ptr [var64],regLo
    adc     dword ptr [var64 + sizeof DWORD],regHi
endm

add64b2Reg macro var64,regLo,regHi
    add     regLo,dword ptr [var64]
    adc     regHi,dword ptr [var64 + sizeof DWORD]
endm

sub64b macro var1lo,var1hi,var2lo,var2hi
    sub     var1lo,var2lo
    sbb     var1hi,var2hi
endm

szSkip macro
    zeroReg eax
    repne   scasb
endm

GetDelta macro Base,Code
    exitm <offset Code - offset Base>
endm

;assigns the difference between addresses of beginVar and endVar
sizeVars macro beginVar,endVar  ;outConst,
    if offset beginVar LT offset endVar
        ;outConst equ (offset endVar-offset beginVar)
        exitm<(offset endVar-offset beginVar)>
    elseif offset beginVar GT offset endVar
        ;outConst equ (offset beginVar-offset endVar)
        exitm<(offset beginVar-offset endVar)>
    else
        ;outConst equ sizeof beginVar
        exitm<sizeof beginVar>
    endif
endm

zeroLocals macro FirstVar
    lea     eax,FirstVar
    mov     edx,ebp
    sub     edx,eax
    invoke  RtlZeroMemory,eax,edx
endm

StoreLocalString macro lpStr:REQ,cbStr:REQ,dwOldEsp
    attr = IsConst(cbStr)
    ifdef dwOldEsp
    	mov     dwOldEsp,esp
    endif
    if attr
        if cbStr GT 2048
        	.ERR <Too big buffer for stack>
        endif
    else
        ifdifi <cbStr>,<ecx>
        ;if cbStr!=ecx then copy contents of cbStr to ecx
        	ifidni <cbStr>,<eax>
        	;if cbStr==eax then use 1-byte "xchg ecx,eax" opcode
        	;else just "mov ecx,x"
        		xchg	ecx,cbStr
        	else
        		mov     ecx,cbStr
        	endif
        endif
    endif
    inc		ecx	;give room for terminating null
    mov     edx,ecx
    alignValue ecx,sizeof DWORD
    sub     esp,ecx
    mov     eax,esp
    invoke  lstrcpyn,eax,lpStr,edx
endm

FreeLocalString macro dwOldEsp
    mov     esp,dwOldEsp
endm

;macro that creates a buffer in stack

;arguments:
;cbBuf:REQ constant that states the size of the buffer
;nAlign - value for alignment of stack buffer
;dwOldEsp - var or reg that will keep old value of stack top if required
;reg - needed if cbBuf is bigger than 2048 bytes for read operations to extend stack
;cnt - is used if cbBuf is bigger than 4096 bytes for loop
MakeLocalBuf macro cbBuf:REQ,nAlign,dwOldEsp,reg,cnt

	ifdef dwOldEsp
		mov		dwOldEsp,esp
	endif
	if cbBuf GT 2048
		times = cbBuf / 2048
		rest = cbBuf MOD 2048
		if times GT 2
			mov		cnt,times
			.repeat
				sub		esp,2048
				mov		eax,[esp]
				dec		cnt
			.until ZERO?
		else
			repeat times
				sub		esp,2048
				mov		eax,[esp]
			endm
		endif
		sub		esp,rest
		mov		eax,[esp]
	else
		sub		esp,cbBuf
	endif
	cons = 0
	ifdef nAlign
		cons = IsConst(nAlign)
	endif
	if cons
	;if ESP address alignment specified then align ESP
		;if IsConst(nAlign)
			;div	ecx
			alignValue	esp,nAlign
		;endif
	endif
endm

FreeLocalBuf macro dwOldEsp
    mov     esp,dwOldEsp
endm

;the following 3 macros make it easier to make and free buffers
makeBuf macro cbBuf
    push    cbBuf
    call    MakeBuff
endm

makeZeroBuf macro cbBuf
    push    cbBuf
    call    MakeZeroBuff
endm

freeBuf macro lpBuf
    push    lpBuf
    call    FreeBuff
endm

;this macro allows to form WNDCLASSEX struct in stack and register the class
;then stack memory is easily freed
RegClass macro style,lpfnWndProc,cbClsExtra,cbWndExtra,hInst,hIcon,hCursor,hbrBackground,lpszMenuName,lpszClassName,hIconSm
    push    hIconSm
    push    lpszClassName
    push    lpszMenuName
    push    hbrBackground
    push    hCursor
    push    hIcon
    push    hInst
    push    cbWndExtra
    push    cbClsExtra
    push    lpfnWndProc
    push    (style)
    push    sizeof WNDCLASSEX   ;cbSize
        invoke  RegisterClassEx,esp
    add     esp,[esp]
endm

return macro val
    regsize EQU @RegSize(val)
    if regsize
    	if regsize EQ 32
			ifdifi <val>,<eax>
				xchg    eax,val
			endif
		else
			.ERR val must be a general-purpose register
    	endif
    else
    	if IsConst(val)
		    ife val ;if val==0==FALSE==NULL
		        zeroReg eax
		    elseif val EQ -1   ;if val==-1==INVALID_HANDLE_VALUE
		        minusOne eax
		    elseif val EQ 1   ;if val==1==TRUE
		        regTrue eax
		    else            ;in other cases
		        mov     eax,val
		    endif
    	else
    		if IsGlobal(val) OR IsLocal(val)
    			mov		eax,val
    		else
    			.ERR <reg> is unknown type of value
    		endif
    	endif
    endif
    ret
endm

retVar macro var
    mov     eax,var
    ret
endm

;retReg macro reg
;	ifdifi <reg>,<eax>
;		xchg    eax,reg
;	endif
;    ret
;endm

;memcopy macro
;    call    CopyStr
;endm

MakeBuf macro cbSize
    push    cbSize
    call    MakeHeapBuf
endm

MakeZeroBuf macro cbSize
    push    cbSize
    call    MakeHeapZeroBuf
endm

FreeBuf macro lpBuf
    push    lpBuf
    call    FreeHeapBuf
endm

ifNullGoto macro reg,dest
    test    reg,reg
    jz      dest
endm

ifNoNullGoto macro reg,dest
    test    reg,reg
    jnz     dest
endm

ifNullVarGoto macro var,dest
    cmp     var,0
    je      dest
endm

MakeColor32_24 macro dest,red,green,blue
    mov     dest,red + (green shl 8) + (blue shl 16)
endm

MakeColor16_555 macro dest,red,green,blue
    mov     dest,((red shr 3) shl 10)+((green shr 3) shl 5) + (blue shr 3)
endm

MakeColor16_565 macro dest,red,green,blue
    mov     dest,((red shr 3) shl 11)+((green shr 3) shl 6) + (blue shr 3)
endm

MakeColor8 macro dest,red,green,blue
    mov     dest,(red shr 5) shl 
endm

MakeColor4 macro dest,red,green,blue
;    mov     dest,(blue shr 6) + (green shr 6)
endm

try macro lpSafePlace:REQ,doSeh:REQ
    lea     edx,[esp-5*4]
    ;we will need future value of ESP when all the data is saved in SEH 
    push    ebp             ;save EBP
    push    edx             ;save ESP that you will get after making the SEH frame
    push    lpSafePlace     ;save EIP of safe place from where we can continue
    xor     ecx,ecx         ;ECX = 0
    push    doSeh           ;ptr to SEH function
    push    fs:[ecx]        ;save ptr to previous SEH
    mov     fs:[ecx],edx    ;save ptr to new SEH in fs:[0]
endm

endtry macro
;preserves EAX, just uses EDX
    xor     edx,edx     ;EDX = 0
    pop     fs:[edx]    ;restore ptr to previous SEH into fs:[0]
    add     esp,4*4
endm

;retValue is the value that will be in EAX when returned from exception filter
except macro retValue
    push    esi
    zeroReg eax
    push    edi
        ife retValue    ;if retValue==0
        elseif retValue EQ -1  ;elseif retValue==-1
            dec     eax 
        elseif retValue EQ TRUE
            inc     eax
        else
            add     eax,retValue
        endif
        mov     esi,[esp+2*4+2*4]   ;get ptr to own SEH_FRAME structure
        mov     edi,[esp+3*4+2*4]   ;ptr to CONTEXT
        mov     (CONTEXT ptr [edi]).regEax,eax
        mov     edx,(SEH_FRAME ptr [esi]).lpSafePlace
        mov     eax,(SEH_FRAME ptr [esi]).regESP
        mov     ecx,(SEH_FRAME ptr [esi]).regEBP
        mov     (CONTEXT ptr [edi]).regEip,edx
        mov     (CONTEXT ptr [edi]).regEsp,eax
        mov     (CONTEXT ptr [edi]).regEbp,ecx
    pop     edi
    pop     esi
    zeroReg eax ;must return 0 == EXCEPTION_CONTINUE_EXECUTION
    ;to reload the context and continue execution from given safe point
    retn
endm

assume fs:nothing   ;this allows us to use fs register in our app

divto1 macro arg,shiftbits
	shr		arg,shiftbits
	.if ZERO?
		inc		arg
	.endif
endm

xcv macro arg:REQ
    mov     eax,OPATTR(arg)
    mov     ecx,@RegSize(arg)
endm

;this macro allows to push given arguments in string
;using the reversed order of arguments
;required for calling STDCALL functions
;it also can work with ADDR directive
;Based on macro from article about macros by Edmond

pushargs macro arglist:VARARG
    ;start with count of arguments = NULL
    args = 0
    ;in "for" loop count arguments and make new string vars for each param in arglist
    for arg,<&arglist>
        ;args++
        args = args + 1
        ;make an array of string vars
        @CatStr(var,%args) textequ <arg>
    endm
    ;n will be used as a counter in repeat loop of macros
    n = args
    repeat n
    ;in each iteration we will do the following:
    ;in argx we will get parameters as strings starting with the last one
    ;if size of the string in argx is bigger than 5 then we should check it for presence of "addr"
    ;if first five chars in string are "addr " or "addr    "
    ;then take the rest of the string as a parameter for LEA and then push it
    ;else just push the parameter
        argx textequ @CatStr(var,%args)
        if @SizeStr(%argx) gt 5 ;; expression "addr ??" must be more than 5 chars
            argaddr textequ @SubStr(%argx,1,5)
            ;next we need to compare the first 5 chars with "addr" followed by a space
            ;and then "addr" followed by a Tab
            ;"ifidni" performs compare of strings case insensitive
            ifidni argaddr,<addr >;"addr" followed by a space
                ;if in @SubStr the length of substring is missing
                ;then it takes the rest of the string
                pushaddr @SubStr(%argx,5)
            else
                ifidni argaddr,<addr	>;"addr" followed by a Tab
                    pushaddr @SubStr(%argx,5)
                else
                    push    argx
                endif
            endif
        else
            push    argx
        endif
        args = args - 1
    endm
endm

pushaddr macro arg:REQ
    ;now we have to find out what type of var or argument we have
    attr = OPATTR(arg)
    if attr AND 20h         ;;if it's a valid argument at all
        attrhi = attr shr 8
        attrlo = attr and 00FFh
        if attrhi
        ;if it's a function reference then attrhi will contain language type of function
        ;else it's a code label
        ;here is the list of language types:
        ;0 - no language type
        ;1 - C
        ;2 - SYSCALL
        ;3 - STDCALL
        ;4 - Pascal
        ;5 - FORTRAN
        ;6 - Basic
            if (attrlo EQ 0A5h) OR (attrlo EQ 0AAh) OR (attrlo EQ 25h) OR (attrlo EQ 2Ah)
            ;if A5h then it is a constant value and it's an address of a function
            ;if AAh then it is a variable that points to an address of a function
            ;if 25h then it is a constant address of a function that has no PROTO declaration
            ;if 2Ah then it is a variable that points to a function without a PROTO declaration
                push    offset arg
            else
                .ERR Invalid DirectX argument
            endif
        else
        ;else it's not a function reference
            if (attr EQ 25h) OR (attr EQ 2Ah)
            ;if global label or global var
                push    offset arg
            elseif (attr EQ 22h) OR (attr EQ 62h)
            ;if local stacked var or register based addressing (eg. [ebx+4])
                lea     eax,arg
                push    eax
            else    ;else show error
            ;actually we should not have anything else here
                .ERR Invalid DirectX argument
            endif
        endif
    else
        .ERR Invalid DirectX argument 
    endif
endm

;MACRO to count timing in ticks for a portion of code
time_begin macro regLo,regHi
	rdtsc
	ifdef regLo AND regHi
		xchg	regLo,eax
		mov		regHi,edx
	else
		push	edx
		push	eax
	endif
endm

time_end macro regLo,regHi
	rdtsc
	ifdef regLo AND regHi
		sub		eax,regLo
		sbb		edx,regHi
	else
		sub		eax,[esp]
		sbb		edx,[esp+4]
		add		esp,8
	endif
endm

time_report macro repStr:REQ,time1:REQ,time2
.const
	szRepStr db <repStr>,0
.data
	szRep	db 256 dup (?)
.code
	invoke	wsprintf,offset szRep,offset szRepStr,time1,time2;offset szOut,offset szFrmt,time1,time2
	invoke	MessageBox,0,offset szRep,offset AppName,MB_OK
endm

;@RegSize macro arg
;
;    res = 0
;    if IsReg(arg) ne FALSE  ;first check if it's a normal register at all
;        argLen = @SizeStr(arg)
;        arg1 textequ @SubStr(<arg>,1,1)
;        if argLen eq 3
;            ifidni arg1,<e>
;                ifidni <arg>,<eax>   ;case insensitive compare
;                    res = 32
;                else
;                    ifidni <arg>,<edx>
;                        res = 32
;                    else
;                        ifidni <arg>,<ecx>
;                            res = 32
;                        else
;                            ifidni <arg>,<ebx>
;                                res = 32
;                            else
;                                ifidni <arg>,<esi>
;                                    res = 32
;                                else
;                                    ifidni <arg>,<edi>
;                                        res = 32
;                                    else
;                                        ifidni <arg>,<esp>
;                                            res = 32
;                                        else
;                                            ifidni <arg>,<ebp>
;                                                res = 32
;                                            else
;                                                ifidni <arg>,<eip>
;                                                    res = 32
;                                                else
;                                                    ifidni <arg>,<efl>
;                                                        res = 32
;                                                    endif
;                                                endif
;                                            endif
;                                        endif
;                                    endif
;                                endif
;                            endif
;                        endif
;                    endif
;                endif
;            else
;                ifidni arg1,<s>
;                    ifidni <arg>,<st0>   ;case insensitive compare
;                        res = 80
;                    else
;                        ifidni <arg>,<st1>
;                            res = 80
;                        else
;                            ifidni <arg>,<st2>
;                                res = 80
;                            else
;                                ifidni <arg>,<st3>
;                                    res = 80
;                                else
;                                    ifidni <arg>,<st4>
;                                        res = 80
;                                    else
;                                        ifidni <arg>,<st5>
;                                            res = 80
;                                        else
;                                            ifidni <arg>,<st6>
;                                                res = 80
;                                            else
;                                                ifidni <arg>,<st7>
;                                                    res = 80
;                                                endif
;                                            endif
;                                        endif
;                                    endif
;                                endif
;                            endif
;                        endif
;                    endif
;                else
;                    ifidni arg1,<m>
;                        ifidni <arg>,<mm0>   ;case insensitive compare
;                            res = 64
;                        else
;                            ifidni <arg>,<mm1>
;                                res = 64
;                            else
;                                ifidni <arg>,<mm2>
;                                    res = 64
;                                else
;                                    ifidni <arg>,<mm3>
;                                        res = 64
;                                    else
;                                        ifidni <arg>,<mm4>
;                                            res = 64
;                                        else
;                                            ifidni <arg>,<mm5>
;                                                res = 64
;                                            else
;                                                ifidni <arg>,<mm6>
;                                                    res = 64
;                                                else
;                                                    ifidni <arg>,<mm7>
;                                                        res = 64
;                                                    endif
;                                                endif
;                                            endif
;                                        endif
;                                    endif
;                                endif
;                            endif
;                        endif
;                    else
;                        ifidni arg1,<d>
;                            ifidni <arg>,<dr0>   ;case insensitive compare
;                                res = 32
;                            else
;                                ifidni <arg>,<dr1>
;                                    res = 32
;                                else
;                                    ifidni <arg>,<dr2>
;                                        res = 32
;                                    else
;                                        ifidni <arg>,<dr3>
;                                            res = 32
;                                        else
;                                            ifidni <arg>,<dr6>
;                                                res = 32
;                                            else
;                                                ifidni <arg>,<dr7>
;                                                    res = 32
;                                                endif
;                                            endif
;                                        endif
;                                    endif
;                                endif
;                            endif
;                        endif
;                    endif
;                endif
;            endif
;        elseif argLen eq 2
;            arg2 textequ @SubStr(<arg>,2,1)
;            ifidni arg1,<a>
;                ifidni arg2,<l>
;                    res = 8 ;al
;                else
;                    ifidni arg2,<h>
;                        res = 8 ;ah
;                    else
;                        ifidni arg2,<x> 
;                            res = 16    ;ax
;                        endif
;                    endif
;                endif
;            else
;                ifidni arg1,<d>
;                    ifidni arg2,<l>;dl
;                        res = 8
;                    else
;                        ifidni arg2,<h>;dh
;                            res = 8
;                        else
;                            ifidni arg2,<x> ;dx
;                                res = 16
;                            else
;                                ifidni arg2,<s>;ds
;                                    res = 16
;                                else
;                                    ifidni arg2,<i>;di
;                                        res = 16
;                                    endif
;                                endif
;                            endif
;                        endif
;                    endif
;                else
;                    ifidni arg1,<c>
;                        ifidni arg2,<l>
;                            res = 8
;                        else
;                            ifidni arg2,<h>
;                                res = 8
;                            else
;                                ifidni arg2,<x> 
;                                    res = 16
;                                endif
;                            endif
;                        endif
;                    else
;                        ifidni arg1,<b>
;                            ifidni arg2,<l>
;                                res = 8
;                            else
;                                ifidni arg2,<h>
;                                    res = 8
;                                else
;                                    ifidni arg2,<x> 
;                                        res = 16
;                                    else
;                                        ifidni arg2,<p> 
;                                            res = 16
;                                        endif
;                                    endif
;                                endif
;                            endif
;                        else
;                            ifidni <arg>,<si>
;                                res = 16
;                            else
;                                ifidni <arg>,<sp>
;                                    res = 16
;                                else
;                                    ifidni <arg>,<es>
;                                        res = 16
;                                    else
;                                        ifidni <arg>,<fs>
;                                            res = 16
;                                        else
;                                            ifidni <arg>,<cs>
;                                                res = 16
;                                            else
;                                                ifidni <arg>,<ss>
;                                                    res = 16
;                                                else
;                                                    ifidni <arg>,<gs>
;                                                        res = 16
;                                                    else
;                                                        ifidni <arg>,<st>   
;                                                            res = 80
;                                                        endif
;                                                    endif
;                                                endif
;                                            endif
;                                        endif
;                                    endif
;                                endif
;                            endif
;                        endif
;                    endif
;                endif
;            endif
;        elseif argLen eq 4
;            ifidni <arg>,<xmm0>
;                res = 128
;            else
;                ifidni <arg>,<xmm1>
;                    res = 128
;                else
;                    ifidni <arg>,<xmm2>
;                        res = 128
;                    else
;                        ifidni <arg>,<xmm3>
;                            res = 128
;                        else
;                            ifidni <arg>,<xmm4>
;                                res = 128
;                            else
;                                ifidni <arg>,<xmm5>
;                                    res = 128
;                                else
;                                    ifidni <arg>,<xmm6>
;                                        res = 128
;                                    else
;                                        ifidni <arg>,<xmm7>
;                                            res = 128
;                                        endif
;                                    endif
;                                endif
;                            endif
;                        endif
;                    endif
;                endif
;            endif
;        elseif argLen eq 5
;            ifidni <arg>,<st(0)>
;                res = 80
;            else
;                ifidni <arg>,<st(1)>
;                    res = 80
;                else
;                    ifidni <arg>,<st(2)>
;                        res = 80
;                    else
;                        ifidni <arg>,<st(3)>
;                            res = 80
;                        else
;                            ifidni <arg>,<st(4)>
;                                res = 80
;                            else
;                                ifidni <arg>,<st(5)>
;                                    res = 80
;                                else
;                                    ifidni <arg>,<st(6)>
;                                        res = 80
;                                    else
;                                        ifidni <arg>,<st(7)>
;                                            res = 80
;                                        endif
;                                    endif
;                                endif
;                            endif
;                        endif
;                    endif
;                endif
;            endif
;        endif
;    endif
;    exitm <res>
;    
;endm

