include windows.inc
include user32.inc
include kernel32.inc
include shell32.inc
include gdi32.inc
include comdlg32.inc
include advapi32.inc

includelib user32.lib
includelib kernel32.lib
includelib shell32.lib
includelib gdi32.lib
includelib comdlg32.lib
includelib advapi32.lib

include macros.inc
include rsconst.inc
include xmmcode.inc
include cubectrl.inc
include fileops.inc


WinMain             PROTO	:DWORD,:DWORD,:DWORD,:DWORD
WndProc             PROTO	:DWORD,:DWORD,:DWORD,:DWORD
SideClrProc			PROTO	:HWND,:UINT,:WPARAM,:LPARAM
GetCrWndProc		PROTO	:HWND,:UINT,:WPARAM,:LPARAM
CubeMove			PROTO	;:LPVOID,:LPVOID,:DWORD
CubeMoveXmm			PROTO	;:LPVOID,:LPVOID,:DWORD
CubeletFromPoint	PROTO	:LPVOID,:DWORD,:DWORD
RotateCubelet		PROTO	:LPVOID,:DWORD,:DWORD
RotateSide			PROTO	:LPVOID,:DWORD,:DWORD
GetColorDlg			PROTO	:HWND,:LPVOID
CommDlgProc			PROTO	:HWND,:UINT,:WPARAM,:LPARAM
EditComment			PROTO	:HWND,:LPVOID
SetsDlgProc			PROTO	:HWND,:UINT,:WPARAM,:LPARAM
HelpDlgProc			PROTO	:HWND,:UINT,:WPARAM,:LPARAM
FindSeqDlgProc		PROTO	:HWND,:UINT,:WPARAM,:LPARAM
LoadSettings		PROTO
SaveSettings		PROTO
SetSequence			PROTO	:LPVOID,:DWORD,:DWORD
SetSeqComm			PROTO	:LPVOID,:DWORD
SetCubeComm			PROTO	:LPVOID,:DWORD
ShowSequence		PROTO	:HWND,:LPVOID,:DWORD
OptimizeSeq			PROTO	:LPVOID,:DWORD
Seq2String			PROTO	:LPVOID,:LPVOID,:DWORD
String2Seq			PROTO	:LPVOID,:LPVOID,:DWORD
Msec2Time			PROTO	:DWORD,:LPVOID
UpdateFindSeqDlg	PROTO	:HWND
FindSeqThread		PROTO	:LPVOID
FindSeq				PROTO	:LPVOID,:LPVOID,:LPVOID,:DWORD,:DWORD
ExeSeq				PROTO	:LPVOID,:DWORD,:LPVOID,:DWORD
IsFast				PROTO

.const

;compiler consts
;IN_THREAD	equ 1

;CPU extentions bit for CPUID
fMMX		equ 00800000h	;it is the same bit that indicates MMX presence
fSSE		equ 02000000h	;and presence of other CPU extentions when CPUID
fSSE2		equ 04000000h	;command is used for identification

;flags for bStats
fCANFAST	equ 01h	;set if SSE2 is supported
fSEARCHING	equ 02h	;is set while the program is searching for a sequence
fUPDATING	equ 04h	;set while updating info about curr sequence
fRECORDING	equ 08h	;set while sequence recording is active
fRECMOVE	equ 10h	;set while we are storing current move
fPAUSED		equ 20h	;set while sequence search thread is suspended

;flags for app settings (Setts.bSets)
fNOFAST		equ 01h	;set if SSE2 is forbidden by the user
fSHOWNUM	equ 02h	;set if square numbers and side letters must be displayed
fNOTHREAD	equ 04h	;set if search for a sequence must not be done in a separate thread
fSIDECLR	equ 08h	;set if cube coloring must be preserved in registry

KEY_PRESSED	equ 80000000h

;cube moves constants
CM_LEFT     equ 0
CM_LEFTX    equ 1
CM_RIGHT    equ 2
CM_RIGHTX   equ 3
CM_TOP      equ 4
CM_TOPX     equ 5
CM_BOTTOM   equ 6
CM_BOTTOMX  equ 7
CM_FRONT    equ 8
CM_FRONTX   equ 9
CM_BACK     equ 10
CM_BACKX    equ 11

;these structures are defined now in cubectrl.inc
;;side structure
;SIDE struct
;    Sqr0 db ?
;    Sqr1 db ?
;    Sqr2 db ?
;    Sqr3 db ?
;    Sqr4 db ?
;    Sqr5 db ?
;    Sqr6 db ?
;    Sqr7 db ?
;SIDE ends
;
;;cube structure
;CUBE struct
;    CubeSide0 SIDE<>
;    CubeSide1 SIDE<>
;    CubeSide2 SIDE<>
;    CubeSide3 SIDE<>
;    CubeSide4 SIDE<>
;    CubeSide5 SIDE<>
;CUBE ends

MAX_MOVES	equ 1024 - ((sizeof CUBE*3)+sizeof DWORD)

SIDECOLOR struct
	rgbColor	RGBQUAD<>
	hBrush		HBRUSH ?
SIDECOLOR ends

FINDSEQ struct
	SrcCube			CUBE<>
	DestCube		CUBE<>
	CurrCube		CUBE<>
	SeqBuffer		db MAX_MOVES dup (?)
	dwPad			dd ?
FINDSEQ ends

BUFF struct
	lpBuf	LPVOID ?
	cbBuf	dd ?
BUFF ends

SETTS struct
	dwMaxMoves			dd ?	;max amount of moves in the sequence
	dwCurrPrior			dd ?	;current thread priority number and offset into dwPriority and szPriority arrays
	dwUpdateFreq		dd ?	;update frequency
	bSets				db ?
SETTS ends

;CONSTANT DATA
align 16
FuncAddr dd offset Left,offset LeftX,offset Right,offset RightX,offset Top,offset TopX
         dd offset Bottom,offset BottomX,offset Front,offset FrontX,offset Back,offset BackX
XmmFuncAddr	dd offset LeftXmm,offset LeftXmmX,offset RightXmm,offset RightXmmX,offset TopXmm,offset TopXmmX
         dd offset BottomXmm,offset BottomXmmX,offset FrontXmm,offset FrontXmmX,offset BackXmm,offset BackXmmX
align 16
OneMoves	db CM_LEFT,CM_LEFTX,CM_RIGHT,CM_RIGHTX,CM_TOP,CM_TOPX,CM_BOTTOM,CM_BOTTOMX,CM_FRONT,CM_FRONTX,CM_BACK,CM_BACKX
align 16
;following is a table with all possible pairs of moves
CNT_TWO_MOVES	equ 132
TwoMoves	db CM_LEFT,CM_LEFT,CM_LEFT,CM_RIGHT,CM_LEFT,CM_RIGHTX,CM_LEFT,CM_TOP,CM_LEFT,CM_TOPX
			db CM_LEFT,CM_BOTTOM,CM_LEFT,CM_BOTTOMX,CM_LEFT,CM_FRONT,CM_LEFT,CM_FRONTX,CM_LEFT,CM_BACK,CM_LEFT,CM_BACKX
			db CM_LEFTX,CM_LEFTX,CM_LEFTX,CM_RIGHT,CM_LEFTX,CM_RIGHTX,CM_LEFTX,CM_TOP,CM_LEFTX,CM_TOPX
			db CM_LEFTX,CM_BOTTOM,CM_LEFTX,CM_BOTTOMX,CM_LEFTX,CM_FRONT,CM_LEFTX,CM_FRONTX,CM_LEFTX,CM_BACK,CM_LEFTX,CM_BACKX
			db CM_RIGHT,CM_LEFT,CM_RIGHT,CM_LEFTX,CM_RIGHT,CM_RIGHT,CM_RIGHT,CM_TOP,CM_RIGHT,CM_TOPX
			db CM_RIGHT,CM_BOTTOM,CM_RIGHT,CM_BOTTOMX,CM_RIGHT,CM_FRONT,CM_RIGHT,CM_FRONTX,CM_RIGHT,CM_BACK,CM_RIGHT,CM_BACKX
			db CM_RIGHTX,CM_LEFT,CM_RIGHTX,CM_LEFTX,CM_RIGHTX,CM_RIGHTX,CM_RIGHTX,CM_TOP,CM_RIGHTX,CM_TOPX
			db CM_RIGHTX,CM_BOTTOM,CM_RIGHTX,CM_BOTTOMX,CM_RIGHTX,CM_FRONT,CM_RIGHTX,CM_FRONTX,CM_RIGHTX,CM_BACK,CM_RIGHTX,CM_BACKX
			db CM_TOP,CM_LEFT,CM_TOP,CM_LEFTX,CM_TOP,CM_RIGHT,CM_TOP,CM_RIGHTX,CM_TOP,CM_TOP
			db CM_TOP,CM_BOTTOM,CM_TOP,CM_BOTTOMX,CM_TOP,CM_FRONT,CM_TOP,CM_FRONTX,CM_TOP,CM_BACK,CM_TOP,CM_BACKX
			db CM_TOPX,CM_LEFT,CM_TOPX,CM_LEFTX,CM_TOPX,CM_RIGHT,CM_TOPX,CM_RIGHTX,CM_TOPX,CM_TOPX
			db CM_TOPX,CM_BOTTOM,CM_TOPX,CM_BOTTOMX,CM_TOPX,CM_FRONT,CM_TOPX,CM_FRONTX,CM_TOPX,CM_BACK,CM_TOPX,CM_BACKX
			db CM_BOTTOM,CM_LEFT,CM_BOTTOM,CM_LEFTX,CM_BOTTOM,CM_RIGHT,CM_BOTTOM,CM_RIGHTX,CM_BOTTOM,CM_TOP,CM_BOTTOM,CM_TOPX
			db CM_BOTTOM,CM_BOTTOM,CM_BOTTOM,CM_FRONT,CM_BOTTOM,CM_FRONTX,CM_BOTTOM,CM_BACK,CM_BOTTOM,CM_BACKX
			db CM_BOTTOMX,CM_LEFT,CM_BOTTOMX,CM_LEFTX,CM_BOTTOMX,CM_RIGHT,CM_BOTTOMX,CM_RIGHTX,CM_BOTTOMX,CM_TOP,CM_BOTTOMX,CM_TOPX
			db CM_BOTTOMX,CM_BOTTOMX,CM_BOTTOMX,CM_FRONT,CM_BOTTOMX,CM_FRONTX,CM_BOTTOMX,CM_BACK,CM_BOTTOMX,CM_BACKX
			db CM_FRONT,CM_LEFT,CM_FRONT,CM_LEFTX,CM_FRONT,CM_RIGHT,CM_FRONT,CM_RIGHTX,CM_FRONT,CM_TOP,CM_FRONT,CM_TOPX
			db CM_FRONT,CM_BOTTOM,CM_FRONT,CM_BOTTOMX,CM_FRONT,CM_FRONT,CM_FRONT,CM_BACK,CM_FRONT,CM_BACKX
			db CM_FRONTX,CM_LEFT,CM_FRONTX,CM_LEFTX,CM_FRONTX,CM_RIGHT,CM_FRONTX,CM_RIGHTX,CM_FRONTX,CM_TOP,CM_FRONTX,CM_TOPX
			db CM_FRONTX,CM_BOTTOM,CM_FRONTX,CM_BOTTOMX,CM_FRONTX,CM_FRONTX,CM_FRONTX,CM_BACK,CM_FRONTX,CM_BACKX
			db CM_BACK,CM_LEFT,CM_BACK,CM_LEFTX,CM_BACK,CM_RIGHT,CM_BACK,CM_RIGHTX,CM_BACK,CM_TOP,CM_BACK,CM_TOPX
			db CM_BACK,CM_BOTTOM,CM_BACK,CM_BOTTOMX,CM_BACK,CM_FRONT,CM_BACK,CM_FRONTX,CM_BACK,CM_BACK
			db CM_BACKX,CM_LEFT,CM_BACKX,CM_LEFTX,CM_BACKX,CM_RIGHT,CM_BACKX,CM_RIGHTX,CM_BACKX,CM_TOP,CM_BACKX,CM_TOPX
			db CM_BACKX,CM_BOTTOM,CM_BACKX,CM_BOTTOMX,CM_BACKX,CM_FRONT,CM_BACKX,CM_FRONTX,CM_BACKX,CM_BACKX
EndMoves 	db -1
;Cube data organization
;Cube has 6 sides. Each side has its nuber. Each side has 9 squares.
;A side is shown as a series 8 squares. The central square is not recorded because
;it never changes or moves and it can be obtained by the number of the side that is
;being worked with. So side numbering is as follows:
;0 - front
;1 - right
;2 - back
;3 - left
;4 - top
;5 - bottom
;constant data
;for the cubes I list positions/colors in the clockwise pattern
;CornerCubes CORNER_CUBE 1 dup (<00h,32h,45h,-1>,<02h,47h,10h,-1>,<05h,50h,37h,-1>,<07h,15h,52h,-1>,\
;                               <20h,12h,42h,-1>,<22h,40h,30h,-1>,<25h,57h,17h,-1>,<27h,35h,55h,-1>)
;EdgeCubes   EDGE_CUBE  1  dup (<0,1,4,6>,<0,3,3,4>,<0,4,1,3>,<0,6,5,1>,\
;                               <1,1,4,4>,<1,6,5,4>,<3,1,4,3>,<3,6,5,3>,\
;                               <2,1,4,1>,<2,3,1,4>,<2,4,3,3>,<2,6,5,6>)
;description of all the cubes on all sides
;there are 4 CUBE_SIDE structures because top and bottom describable cubes are already mentioned on sides
;FullCube0 CUBE_SIDE <<3,2,4,5>,<4,6,-1>,<4,7,1,0>,<3,4,-1>,<1,3,-1>,<5,0,3,7>,<5,2,-1>,<1,5,5,2>>
;FullCube1 CUBE_SIDE <<0,0,0,0>,<4,4,-1>,<0,0,0,0>,<0,0,0>,<0,0,0>,<0,0,0,0>,<5,4,-1>,<0,0,0,0>>
;FullCube2 CUBE_SIDE <<1,2,4,2>,<4,1,-1>,<4,0,3,0>,<1,4,-1>,<3,3,-1>,<5,7,1,7>,<5,6,-1>,<3,5,5,5>>
;FullCube3 CUBE_SIDE <<0,0,0,0>,<4,3,-1>,<0,0,0,0>,<0,0,0>,<0,0,0>,<0,0,0,0>,<5,3,-1>,<0,0,0,0>>
;EndOfFullCube       dw -1
;FullCube0 CUBE_SIDE 1 dup  (<<00h,32h,45h,-1>,<0,1,4,6>,<02h,47h,10h,-1>,<0,3,3,4>,<0,4,1,3>,<05h,50h,37h,-1>,<0,6,5,2>,<07h,15h,52h,-1>>,\
;                            <<10h,02h,47h,-1>,<1,1,4,4>,<12h,42h,20h,-1>,<1,3,0,4>,<1,4,2,3>,<15h,52h,07h,-1>,<1,6,5,4>,<17h,25h,57h,-1>>,\
;                            <<20h,12h,42h,-1>,<2,1,4,1>,<22h,40h,30h,-1>,<2,3,1,4>,<2,4,3,3>,<25h,57h,17h,-1>,<2,6,5,6>,<27h,35h,55h,-1>>)
;FullCube1 CUBE_SIDE 1 dup  (<<30h,22h,40h,-1>,<3,1,4,3>,<32h,45h,00h,-1>,<3,3,2,4>,<3,4,0,3>,<35h,55h,27h,-1>,<3,6,5,3>,<37h,05h,50h,-1>>,\
;                            <<40h,30h,22h,-1>,<4,1,2,1>,<42h,20h,12h,-1>,<4,3,3,1>,<4,4,1,1>,<45h,00h,32h,-1>,<4,6,0,1>,<47h,10h,02h,-1>>,\
;                            <<50h,37h,05h,-1>,<5,1,0,6>,<52h,07h,15h,-1>,<5,3,3,6>,<5,4,1,6>,<55h,27h,35h,-1>,<5,6,2,6>,<57h,17h,25h,-1>>)

;cube sides are numbered the following sequence:
;F,R,B,L,T,U
;Any color for cube will be this:
ANY_COLOR	equ 0FFFFFFFFh

align 16;Assembled cube
;assembled cube as a CUBE structure
AsmCube  SIDE <0,0,0,0,0,0,0,0>,<1,1,1,1,1,1,1,1>,<2,2,2,2,2,2,2,2>,<3,3,3,3,3,3,3,3>,<4,4,4,4,4,4,4,4>,<5,5,5,5,5,5,5,5>
;XCube  db 0,1,2,3,4,5,6,7,0,1,2,3,4,5,6,7,0,1,2,3,4,5,6,7,0,1,2,3,4,5,6,7,0,1,2,3,4,5,6,7,0,1,2,3,4,5,6,7
;align 16
;dbSide 	db 00h,01h,02h,03h,04h,05h,06h,07h
;		db 10h,11h,12h,13h,14h,15h,16h,17h
;		db 20h,21h,22h,23h,24h,25h,26h,27h
;		db 30h,31h,32h,33h,34h,35h,36h,37h
;		db 40h,41h,42h,43h,44h,45h,46h,47h
;		db 50h,51h,52h,53h,54h,55h,56h,57h
;		db 60h,61h,62h,63h,64h,65h,66h,67h
;		db 70h,71h,72h,73h,74h,75h,76h,77h
;dbConst db 0,0,0,0,0,0,0FFh,0FFh
CustColors 	dd 	00000000h,00FFFFFFh,000000FFh,0000FF00h,00FF0000h,0000FFFFh,00FFFF00h,00808080h,\
				00000080h,00008000h,00800000h,00008080h,00808000h,00A0A0A0h,00C04040h,00C0C0C0h

;thread priorities
dwPriority	dd THREAD_PRIORITY_TIME_CRITICAL,THREAD_PRIORITY_HIGHEST,\
				THREAD_PRIORITY_NORMAL,THREAD_PRIORITY_LOWEST,THREAD_PRIORITY_IDLE

;local const data
align 8
;move chars
cMoveChars	db "L","R","T","U","F","B"
align 8
;convert side number to the move that is done on the side
Side2Move	db CM_FRONT,CM_RIGHT,CM_BACK,CM_LEFT,CM_TOP,CM_BOTTOM
NoMove		db -1	;it must always follow Side2Move!

align 8
szPriority	db "Time Critical",0,"High",0,"Normal",0,"Low",0,"Idle",0

align 8
ClassName	db 'DLGCLASS'
szEmpty		db 0
CubeClass	db "CubeCtlClass",0
AppName  	db "Rubik's Cube Emulator",0
szPause		db "Pause",0
szRestart	db "Restart",0
sz2strings	db "%s - %s",0
szSrcCb		db "Source Cube",0
szDstCb		db "Destination Cube",0
szSeqCom	db "Sequence Comment Dialog",0
szCubeCom	db "Cube Comment Dialog",0
szRegPath	db "Software\SolidCode\RubikCubeEmulator",0
szRegSets	db "Settings",0
szRegClrs	db "Colors",0
szRegSz		db "REG_SZ",0
AboutMsg	db "Rubik's Cube Emulator by SolidCode. 2006.",0
Need586		db "This program requires Pentium processor or higher.",13,10,"Press OK to exit.",0
szHighPrior	db "This priority for thread is too high.",13,10
			db "In most cases it will not encrease the speed significantly",13,10
			db "but may cause problems with window responce and system performance.",0
szNotFound	db "No sequence was found for current limit of moves (%lu).",13,10
			db "Time elapsed: %s",13,10
			db 'You can try to encrease "Maximum amount of moves" in Options Dialog.',0
szFoundSeq	db "The required sequence was found.",13,10
			db "Time elapsed: %s",13,10
			db "Length of the sequence: %lu moves.",13,10
			db "Sequence string (max first 120 chars shown):",13,10
			db "%s",13,10
			db "Press OK to accept the sequence or Cancel to discard results.",0
szTime		db "%lu hours; %02lu minutes; %02lu seconds; %03lu milliseconds",0
szSecs		db "%lu,%03lu sec.",0
szStopRec	db "Sequence recording is about to be stopped.",13,10
			db "Would you like to save recorded sequence?",13,10
			db 'Press "Yes" to stop sequence recording and save current sequence to file.',13,10
			db 'Press "No" to stop sequence recording and discard current sequence.',13,10
			db 'Press "Cancel" to continue recording.',0
szHelp		db "Introduction",13,10;,13,10
			db "This application is designed as a simple emulator of Rubik's Cube. "
			db "While there is a lot of better Rubik's Cube emulators out there, "
			db "the primary intent for this one is to implement the fastest possible "
			db "sequence search algo. That is when we provide the original state of "
			db "the cube and the desired one, then the program starts searching for a "
			db "sequence of cube moves that would end in the desired state transition. "
			db "The search begins with one-move sequences. If none of the possible "
			db "one-move sequences can perform the transition, we increment the amount of moves "
			db "and search through all the two-move sequences. The search continues until "
			db "the desired sequence is found or moves limit is reached.",13,10
			db "This search is implemented in two versions:",13,10
			db "1. supported on all Pentium (586) processor;",13,10
			db "2. supported on Pentium IV processors with SSE2 extentions.",13,10
			db "Method 2 uses SSE and SSE2 instructions (SIMD technology). "
			db "Use of XMM registers allows to keep the entire cube state "
			db "and do computation in the registers only, which minimizes memory access. "
			db "The instructions were interleaved as much as possible to make use of Hyper-Threading Technology. ",13,10
			db 13,10
			db "Sequences",13,10
			db 'Move sequences can be loaded and saved using Sequence menu. Under "Settings" '
			db "you can store comment text that will be attached to the sequence. Current "
			db "sequence is displayed in the text field under source and destination cubes. "
			db 'You can execute current sequence by pressing "Execute" button or the "Enter" key, '
			db "if keyboard focus is in the text field at the moment.",13,10
			db "There are 6 sides to a cube:",13,10
			db "L - Left",13,10
			db "R - Right",13,10
			db "T - Top",13,10
			db "U - Underside (bottom)",13,10
			db "F - Front",13,10
			db "B - Back",13,10
			db "Each side can be rotated by 90 degrees. If the side is rotated clockwise (CW), "
			db "the move is written as the corresponding side letter. If the side is rotated counter "
			db "clockwise (CCW), the move is written as the letter followed by an apostrophy ('). "
			db 'If the side is rotated 180 degrees, then it is recorded as the letter followed by '
			db 'a quotation mark ("). You can write down or paste move sequences.',13,10
			db "You can also record side rotations in the text field. To do this go to "
			db '"Sequences->Record Sequence" in the main menu. The text field background will turn '
			db "yellow indicating that sequence recording mode is activated. When you need to exit "
			db 'the mode, select "Sequences->Stop Recording". Remember that only destination cube '
			db "side rotations are recorded.",13,10
			db 'Sequences can be saved as "CSQ" files. These files can be loaded into the program as well.',13,10
			db 13,10
			db "Cube State",13,10
			db 'The state of both source and destination cubes can be saved to "CSF" and loaded into the '
			db 'program. Any of the cubes can be assebled through "Assemble Cube" menu items. Each cube '
			db 'can also be copied into the other cube through "Copy Cube" menu item.',13,10
			db 13,10
			db "Actions",13,10
			db '"Find Sequence" menu item allows you to find a sequence of moves that will take the '
			db 'source cube state and result in the destination cube state. All the options for this '
			db 'process are covered in the Options article.',13,10
			db '"Execute Sequence" menu item takes the source cube state and performs the sequence of '
			db 'moves from the text field. The resulting state is saved into the destination cube.',13,10
			db "Sometimes move sequences can eliminate each other. E.g. if a side is rotated 90 degrees CW "
			db "and then it is rotated 90 CCW. To eliminate these extra moves you can use menu "
			db '"Actions->Optimize Sequence".',13,10
			db '"Swap Cubes" menu item allows to swap cube states of source and destination cubes.',13,10
			db '"Default Coloring" loads default colors for sides.',13,10
			db 13,10
			db '"Settings"',13,10
			db 'Menu item "Sides Coloring..." shows a dialog box that allows you to set colors '
			db 'for any of the sides in the cube.',13,10
			db '"Sequence Comment..." and "Cube Comment..." menu items allow you to specify comments '
			db 'that will be saved into the sequence or cube state files respectively.',13,10
			db '"Options..." menu item opens the "Emulator Settings" dialog box.',13,10
			db '"Emulator Settings"',13,10
			db '"Maximum amount of moves" specifies the limit that can be accepted by the program. '
			db 'It is also the limit for "Find Sequence". The biggest value acceptable is 876.',13,10
			db '"Update frequency" shows how often the program will update "Sequence Search" dialog. '
			db 'This dialog displays sequence search process statistics. '
			db 'The value is changed by dragging the slider on the right.',13,10
			db '"Search thread priority" specifies the priority that will be set for the execution '
			db 'thread that performs sequence search. In most cases it is best to leave "Idle". Other '
			db 'values can encrease search speed under certain conditions. "Time Critical" and "High" '
			db 'values are not recommended. In most cases they may negatively affect the system performance '
			db 'and window responce.',13,10
			db 'Checkbox "Disable threaded search" forces the program to search for a sequence in the main '
			db 'thread. It prevents the main window from responding to user input. Checking this box '
			db 'is not recommended.',13,10
			db '"Disable SSE2 acceleration" allows the user to switch off use of SSE2 algorith.',13,10
			db '"Display side letters and square numbers" allows the user to see number of each square '
			db 'and letters that represent each side.',13,10
			db '"Preserve side coloring" tells the program to save current cube colors into the registry '
			db 'along with other settings.',13,10
			db 'Button "Save Settings" saves all current settings into the registry.',13,10
			db 'Button "OK" applies current settings and closes "Emulator Settings" dialog.',13,10
			db 'Button "Cancel" closes the dialog without applying the settings.',13,10
			db 'Menu item "Save Settings" saves all current settings into the registry.',13,10
			db 13,10
			db '"Help"',13,10
			db 'Menu item "Help" displays current help dialog.',13,10
			db '"About" provides you with basic info about the application and its author.'
			db 0


;-----------------------------------------------------------------------------------
;variable data
.data?
align 16
FindSequence		FINDSEQ<>
nCurrMoves			dd ?
align 16
Setts				SETTS<>
align 4
bStats				db ?
align 4
hEdBrush			dd ?

align 16
arrSideColors		RGBQUAD <>,<>,<>,<>,<>,<>	;colors for sides will be kept here
UndefSideColor		RGBQUAD <>	;this must always follow arrSideColors!
;cubes must be drawn this way:
;first fill all middle cubes on sides; then fill surrounding cubes
align 16
SrcCube				CUBE<>
SrcRect				RECT<>
hSrcWnd				HWND ?
lpfnOrigSrcProc		LPVOID ?

DestCube			CUBE<>
DestRect			RECT<>
hDestWnd			HWND ?
lpfnOrigDestProc	LPVOID ?

align 16	;current sequence vars
SeqBuf				BUFF<>	;SeqBuf array
cntMoves			dd ?	;amount of moves in the array
hProcHeap			dd ?
szComment			BUFF <>
szCubeComm			BUFF <>

align 16
arrTempColors		SIDECOLOR <>,<>,<>,<>,<>,<>	;this is a temporary storage for side colors and brushes
TempUndefColor		SIDECOLOR <>	;this must always follow arrTempColors!
arrCustColor		RGBQUAD <>,<>,<>,<>,<>,<>,<>,<>,<>,<>,<>,<>,<>,<>,<>,<>
lpfnOldEdit			LPVOID ?
hInstance			dd ?
CommandLine			dd ?
hWnd				dd ?
hIcon				dd ?
FindSeqThreadID		dd ?	;thread ID
hFindSeqThread		dd ?	;handle to thread
hFindSeqDlg			dd ?
IDT_FINDSEQ			equ 20000
hTimer				dd ?
IDT_PFINDSEQ		equ 20001
hProgTimer			dd ?
dwStartTime			dd ?
dwStillTime			dd ?
align 16	;global strings
szSeqFileName		db 512 dup (?)
szSrcCubeFileName	db 512 dup (?)
szDstCubeFileName	db 512 dup (?)
szCaption			db 512 dup (?)
szString			db 1024 dup (?)
SeqBuffer			db 1024 dup (?)

;---------------------------------------------------------------------------------------

;FindSeqData			FINDSEQ<>
;;array for function pointers
;;FuncPtrs			dd 1024 dup (?)
;CurrCube            CUBE<>
;;Moves				MOVES<>
;align 16
;SeqBuffer   		db 1024 dup (?) ;this is a buffer for the sequence
;;dwPad				dd 4 dup (?)
;SeqBufferOpt   		db 1024 dup (?) ;this is a buffer for the sequence
;;dwPad0				dd ?
;SeqBufferPrev  		db 1024 dup (?) ;this is a buffer for the sequence
;;dwPad1				dd 4 dup (?)



;Old junk:


;Rubik's Cube structures
;CORNER_CUBE struct
;    ;Side0 is described by the position at which the structure is located in a cube
;    Side1Color  db ?
;    Side1Pos    db ?
;    Side2Color  db ?
;    Side2Pos    db ?
;CORNER_CUBE ends

;EDGE_CUBE struct
;    ;Edge0 is described by the position of the structure in a cube
;    Edge1Color  db ?
;    Edge1Pos    db ?
;    CornerID    dw ?    ;must always be -1, indicates that it is an edge cube described
;EDGE_CUBE ends

;CUBE_SIDE struct
;    Cube0 CORNER_CUBE<>
;    Cube1 EDGE_CUBE<>
;    Cube2 CORNER_CUBE<>
;    Cube3 EDGE_CUBE<>
;    Cube4 EDGE_CUBE<>
;    Cube5 CORNER_CUBE<>
;    Cube6 EDGE_CUBE<>
;    Cube7 CORNER_CUBE<>
;CUBE_SIDE ends

;CONSTS struct
;	dq067 	dq ?
;	dq067h	dq ?
;	dq234	dq ?
;	dq234h	dq ?
;CONSTS ends


;MmxFuncAddr	dd offset LeftMm,offset LeftMmX,offset RightMm,offset RightMmX,offset TopMm,offset TopMmX
;			dd offset BottomMm,offset BottomMmX,offset FrontMm,offset FrontMmX,offset BackMm,offset BackMmX



;FINDSEQ struct
;	arrMoveAddr		dd MAX_MOVES dup (?)
;	ddPad			dd 4 dup (?)
;	SeqBuffer		db MAX_MOVES dup (?)
;	dwPad			db 4 dup (?)
;	SeqBufferOpt   	db MAX_MOVES dup (?) ;this is a buffer for the sequence
;	dwPad0			db 4 dup (?)
;	SeqBufferPrev  	db MAX_MOVES dup (?) ;this is a buffer for the sequence
;	dwPad1			db 4 dup (?)
;	CurrCube		CUBE<>
;	SrcCube			CUBE<>
;	DestCube		CUBE<>
;	dwPadF			db MAX_MOVES+4 - sizeof CUBE * 3 dup (?)
;FINDSEQ ends



;cntTwoMoves	dd CNT_TWO_MOVES	;amount of words in the previous array; amount of possible pairs of moves' combinations
;Consts	CONSTS<0FFFF0000000000FFh,0FFFF0000000000FFh>;,0000000FFFFFF0000h,0000000FFFFFF0000h>;,\
			;<00000FFFFFFFFFF00h,0000FFFFFFFFFF00h>,\
			;<0FFFFFF000000FFFFh,0FFFFFF000000FFFFh>>
;X_MOVES equ 10
;align 16
;Moves0  db CM_LEFT,CM_LEFT,CM_LEFT,CM_LEFT,CM_LEFT,CM_LEFT,CM_LEFT,CM_LEFT
;Moves1  db CM_LEFTX,CM_LEFTX,CM_LEFTX,CM_LEFTX,CM_LEFTX,CM_LEFTX,CM_LEFTX,CM_LEFTX
;Moves2  db CM_RIGHT,CM_RIGHT,CM_RIGHT,CM_RIGHT,CM_RIGHT,CM_RIGHT,CM_RIGHT,CM_RIGHT
;Moves3  db CM_RIGHTX,CM_RIGHTX,CM_RIGHTX,CM_RIGHTX,CM_RIGHTX,CM_RIGHTX,CM_RIGHTX,CM_RIGHTX
;Moves4  db CM_TOP,CM_TOP,CM_TOP,CM_TOP,CM_TOP,CM_TOP,CM_TOP,CM_TOP
;Moves5  db CM_TOPX,CM_TOPX,CM_TOPX,CM_TOPX,CM_TOPX,CM_TOPX,CM_TOPX,CM_TOPX
;Moves6  db CM_BOTTOM,CM_BOTTOM,CM_BOTTOM,CM_BOTTOM,CM_BOTTOM,CM_BOTTOM,CM_BOTTOM,CM_BOTTOM
;Moves7  db CM_BOTTOMX,CM_BOTTOMX,CM_BOTTOMX,CM_BOTTOMX,CM_BOTTOMX,CM_BOTTOMX,CM_BOTTOMX,CM_BOTTOMX
;Moves8  db CM_FRONT,CM_FRONT,CM_FRONT,CM_FRONT,CM_FRONT,CM_FRONT,CM_FRONT,CM_FRONT
;Moves9  db CM_FRONTX,CM_FRONTX,CM_FRONTX,CM_FRONTX,CM_FRONTX,CM_FRONTX,CM_FRONTX,CM_FRONTX
;Moves10 db CM_BACK,CM_BACK,CM_BACK,CM_BACK,CM_BACK,CM_BACK,CM_BACK,CM_BACK
;Moves11 db CM_BACKX,CM_BACKX,CM_BACKX,CM_BACKX,CM_BACKX,CM_BACKX,CM_BACKX,CM_BACKX
;CycleMove0 db CM_LEFT,CM_FRONTX,CM_LEFTX,CM_FRONT,CM_LEFT,CM_FRONTX,CM_LEFTX,CM_FRONT,CM_LEFT,CM_FRONTX,CM_LEFTX,CM_FRONT,CM_LEFT,CM_FRONTX,CM_LEFTX,CM_FRONT,CM_LEFT,CM_FRONTX,CM_LEFTX,CM_FRONT,CM_LEFT,CM_FRONTX,CM_LEFTX,CM_FRONT ;6
;CycleMove1 db CM_LEFT,CM_RIGHT,CM_FRONT,CM_BACK,CM_LEFT,CM_RIGHT,CM_FRONT,CM_BACK,CM_LEFT,CM_RIGHT,CM_FRONT,CM_BACK
