;here all 6 sides are kept in MMX registers
;computation is done with MMX and XMM registers

;--------------------------------------------------------------------------------

;PURPOSE
;start with cube state in ESI, use sequence from EDI
;and execute ECX moves on the cube 

;IN
;ESI - ptr to CUBE struct
;EDI - ptr to sequence buffer
;ECX - count of moves to execute in the sequence buffer

;OUT
;the resulting cube is stored in XMM0,XMM1,XMM2

;REMARKS
;regs xmm0,xmm1,xmm2 keep the resulting cube sides
		;reg xmm7 keeps !dq234:!dq067
		;reg xmm6 keeps dq067:dq067
		;other regs (xmm3,xmm4,xmm5) can be used to perform calculations
;
;CubeMoveXmm assumes that XMM6 and XMM7 have been prepared
;beforehand using xmm_setconst macro.

align 16
CubeMoveXmm proc ;uses ebx ;lpCube:LPVOID,lpMoves:LPVOID,cbMoves:DWORD
	
	assume esi:ptr CUBE
	;assume is done for the functions themselves
	;ESI is always lpCube
	;EDI is lpMoves
	movdqa	xmm0,[esi].CubeSide0
	movdqa	xmm1,[esi].CubeSide2
	movdqa	xmm2,[esi].CubeSide4
	movc	eax,0
	sub		edx,edx	;use EDX as offset into lpMoves in EDI
align 16
	.repeat
		mov		al,[edi+edx]
		lea		esp,[esp]	;filler to avoid stall and to align code
		mov		eax,[eax*sizeof DWORD + offset XmmFuncAddr]
		inc		edx
		call	eax
		movc	eax,0
	.until edx>=ecx
	ret

;	;mov		esi,offset dbSide;lpCube	;offset dbSide;
;	;mov		edi,lpMoves
;	;mov		ecx,cbMoves
;	assume ebx:ptr FINDSEQ
;	lea		edx,[ebx].DestCube
;	;from now on EDX may not be changed
;		mov		esi,ebx	;lea		esi,[ebx].SrcCube
;		assume esi:ptr CUBE
;		;ESI is always lpCube
;		;EDI is lpMoves
;		movdqa	xmm0,[esi].CubeSide0
;		movc	eax,0
;		xmm_mov	xmm1,[esi].CubeSide2
;		lea		ebx,[edi+ecx]	;calc lpEnd for the latter loop
;		xmm_mov	xmm2,[esi].CubeSide4
;		;assume is done for the functions themselves
;align 16
;		.repeat
;			mov		al,[edi]
;			mov		eax,[eax*sizeof DWORD + offset XmmFuncAddr]
;			add		edi,1
;			call	eax
;			movc	eax,0
;		.until edi>=ebx
;		;save data to memory
;;		movdqa	[esi].CubeSide0,xmm0
;;		movdqa	[esi].CubeSide2,xmm1
;;		movdqa	[esi].CubeSide4,xmm2
;	mov		edi,edx
;	assume edi:ptr CUBE
;	;compare cubes with SSE2 too
;	pcmpeqb	xmm0,[edi].CubeSide0
;	pcmpeqb	xmm1,[edi].CubeSide2
;	pcmpeqb	xmm2,[edi].CubeSide4
;	pand	xmm0,xmm1
;	movc	eax,FALSE
;	pand	xmm0,xmm2
;	;if ax==-1 then cubes match so return TRUE
;	;else cubes don't match so return FALSE
;	pmovmskb	edx,xmm0
;	add		dx,1
;	adc		eax,eax
;	ret


;			movq	mm0,[esi].CubeSide0
;			mmx_mov	mm1,[esi].CubeSide1
;			mmx_mov	mm2,[esi].CubeSide2
;			movq	mm3,[esi].CubeSide3
;			mmx_mov	mm4,[esi].CubeSide4
;			mmx_mov	mm5,[esi].CubeSide5

;				movc	edx,0
;				mov		esi,offset Consts
;				;assume is done for the functions themselves
;				mov		dl,sizeof Consts / sizeof OWORD
;				;put Consts array into cache
;				.repeat
;					mov		eax,dword ptr [esi]
;					add		esi,sizeof OWORD
;					sub		edx,1
;				.until ZERO?
;				;restore ESI to ptr Consts
;				add		esi,-sizeof Consts

;			movq	[esi].CubeSide0,mm0
;			movq	[esi].CubeSide1,mm1
;			movq	[esi].CubeSide2,mm2
;			movq	[esi].CubeSide3,mm3
;			movq	[esi].CubeSide4,mm4
;			movq	[esi].CubeSide5,mm5

CubeMoveXmm endp

;the optimization principle for the SSE2 code was majorly based on the following:
;	combining instructions that are executed in different execution units
;	breaking register dependencies:
;		writing to same register in neighboring instructions
;		reading register after a write to this same register

align 16
LeftXmm::
;	00->50	40->00	22->46	50->24
;	06->56	46->06	23->47	56->22
;	07->57	47->07	24->40	57->23
;	3 rotate left

					xmm_xchgLq	xmm4,xmm1
			xmm_xchg	xmm3,xmm2
					xmm_lo2hi	xmm4,xmm0
	pand	xmm2,xmm7	;!.dq067
							xmm_mov	xmm5,xmm4
			pand	xmm3,xmm6	;.dq067
					pand	xmm4,xmm7	;!.dq067
			xmm_xchgLq	xmm3,xmm3
							pand	xmm5,xmm6	;.dq067
			xmm_xchgLq	xmm4,xmm4
	xmm_rolHq	xmm1,xmm1
					por		xmm4,xmm3
	por		xmm2,xmm5
					xmm_hi2lo	xmm0,xmm4
					xmm_lo2lo	xmm1,xmm4
	retn

align 16
LeftXmmX::
;00>40	50>00   22>56   40>24
;06>46	56>06   23>57   46>22
;07>47	57>07   24>50   47>23
;3 shr 16
;xmm7 = !067		;xmm6 = 067
					xmm_xchgLq	xmm4,xmm1	;xmm4.lo = xchg(side2)
			xmm_mov	xmm3,xmm2	;xmm3 = xmm2 = side5:side4
					xmm_lo2hi	xmm4,xmm0	;xmm4.hi = side0
	pand	xmm2,xmm7	;xmm2 and !067:!067
							xmm_xchg	xmm5,xmm4	;xmm5 = side2:side0
			pand	xmm3,xmm6	;xmm3 and .dq067:.dq067
					pand	xmm4,xmm7	;!.dq067
			xmm_xchgLq	xmm3,xmm3
							pand	xmm5,xmm6	;.dq067
			xmm_xchgLq	xmm4,xmm4
	xmm_rorHq	xmm1,xmm1	;side3 shr 16bit
					por		xmm4,xmm3
	por		xmm2,xmm5
					xmm_hi2lo	xmm0,xmm4
					xmm_lo2lo	xmm1,xmm4
	retn

align 16
RightXmm::
;02>42	52>02   20>54   42>26
;03>43	53>03   26>52   43>27
;04>44	54>04   27>53   44>20
;1 shl 16
					xmm_xchgLq	xmm4,xmm1
	xmm_dxchg	xmm6,xmm6
					xmm_lo2hi	xmm4,xmm0
			xmm_mov	xmm3,xmm2
					xmm_xchg	xmm5,xmm4
			pand	xmm3,xmm6	;234234
	xmm_dxchg	xmm7,xmm7
					pand	xmm5,xmm6	;234234
	pand	xmm2,xmm7
	xmm_rolHq	xmm0,xmm0
					pand	xmm4,xmm7	;!234234
	xmm_dxchg	xmm6,xmm6
					por		xmm4,xmm3
	xmm_dxchg	xmm7,xmm7
					;shufps	xmm1,xmm4,shuf(0,1,3,2)
					xmm_lo2lo	xmm1,xmm4
	por		xmm2,xmm5
					xmm_hi2lo	xmm0,xmm4
					;xmm_xchg	xmm1,xmm1
					xmm_xchgLq	xmm1,xmm1
	retn

align 16
RightXmmX::		;check it!!!
;02>52	42>02   20>44   52>26
;03>53	43>03   26>42   53>27
;04>54	44>04   27>43   54>20
;1 shr 16
					xmm_xchgLq	xmm4,xmm1	;x4.lo = xchg(side2)
	xmm_dxchg	xmm6,xmm6	;x6 = src234 from src067
					xmm_lo2hi	xmm4,xmm0	;x4.hi = side0
			xmm_xchg	xmm3,xmm2	;xmm_mov	xmm3,xmm2
					xmm_mov	xmm5,xmm4
			pand	xmm3,xmm6	;234234
	xmm_dxchg	xmm7,xmm7
					pand	xmm5,xmm6	;234234
	pand	xmm2,xmm7
	xmm_rorHq	xmm0,xmm0
					pand	xmm4,xmm7	;!234234
	xmm_dxchg	xmm6,xmm6
					por		xmm4,xmm3
	xmm_dxchg	xmm7,xmm7
					;shufps	xmm1,xmm4,shuf(0,1,3,2)
					xmm_lo2lo	xmm1,xmm4
	por		xmm2,xmm5
					xmm_hi2lo	xmm0,xmm4
					;xmm_xchg	xmm1,xmm1
					xmm_xchgLq	xmm1,xmm1
	retn

align 16
TopXmm::
;00>30	10>00   20>10   30>20
;01>31	11>01   21>11   31>21
;02>32	12>02   22>12   32>22
;4 shr 16
	pshufd	xmm3,xmm0,shuf(3,1,2,0);11011000b	;ready
	xmm_minus1	xmm5
	shufps	xmm3,xmm1,shuf(2,0,1,0);10000100b
	pslld	xmm5,3*8
	xmm_ror	xmm4,xmm3
	pand	xmm3,xmm5
	pandn	xmm5,xmm4
	xmm_rolLq	xmm2,xmm2
	por		xmm3,xmm5
	shufps	xmm0,xmm3,shuf(1,0,3,1);01001101b
	shufps	xmm1,xmm3,shuf(3,2,3,1);11101101b
	pshufd	xmm0,xmm0,shuf(1,3,0,2);01110010b
	pshufd	xmm1,xmm1,shuf(1,3,0,2);01110010b
	retn

align 16
TopXmmX::
	pshufd	xmm3,xmm0,shuf(3,1,2,0);11011000b	;ready
	xmm_minus1	xmm5
	shufps	xmm3,xmm1,shuf(2,0,1,0);10000100b
	pslld	xmm5,3*8
	xmm_rol	xmm4,xmm3
	pand	xmm3,xmm5
	pandn	xmm5,xmm4
	xmm_rorLq	xmm2,xmm2
	por		xmm3,xmm5
	shufps	xmm0,xmm3,shuf(1,0,3,1);01001101b
	shufps	xmm1,xmm3,shuf(3,2,3,1);11101101b
	pshufd	xmm0,xmm0,shuf(1,3,0,2);01110010b
	pshufd	xmm1,xmm1,shuf(1,3,0,2);01110010b
	retn

align 16
BottomXmm::;check the following two moves!!!
	pshufd	xmm3,xmm0,shuf(2,0,3,1);10001101b
	xmm_minus1	xmm5
	shufps	xmm3,xmm1,shuf(3,1,1,0);11010100b
	pslld	xmm5,3*8
	xmm_rol	xmm4,xmm3
	pand	xmm3,xmm5
	pandn	xmm5,xmm4
	xmm_rolHq	xmm2,xmm2
	por		xmm3,xmm5
	shufps	xmm0,xmm3,shuf(1,0,2,0);01001000b
	shufps	xmm1,xmm3,shuf(3,2,2,0);11101000b
	pshufd	xmm0,xmm0,shuf(3,1,2,0);11011000b
	pshufd	xmm1,xmm1,shuf(3,1,2,0);11011000b
	retn

align 16
BottomXmmX::
	pshufd	xmm3,xmm0,shuf(2,0,3,1);10001101b
	;we need to save hi dwords from side0 and side1 into dwords 0 and 1 in xmm3, the rest of xmm3 does not matter
	xmm_minus1	xmm5
	shufps	xmm3,xmm1,shuf(3,1,1,0);11010100b
	;save hi dwords from side2 and side3 into dwords 2 and 3 in xmm3 while preserving lo dwords (0 and 1)
	pslld	xmm5,3*8	;clear first 3 bytes with 0 and highest byte must remain FFh
	xmm_ror	xmm4,xmm3	;xmm4 must have xmm3 rotated by 4 bytes to right
	pand	xmm3,xmm5	;clear dest bytes in xmm3
	pandn	xmm5,xmm4	;not(xmm5) and xmm4 clear src highest byte in xmm4
	xmm_rorHq	xmm2,xmm2	;rotate side 5 by 2 bytes to the right
	por		xmm3,xmm5	;in xmm3 combine bytes
	;now we have all the dwords ready, got to save them back to sides 0 - 1
	shufps	xmm0,xmm3,shuf(1,0,2,0);01001000b
	;preserve dwords 0 and 2 into the low qword and save new dwords 1 and 3 into xmm0
	shufps	xmm1,xmm3,shuf(3,2,2,0);11101000b
	;preserve dwords 0 and 2 into the low qword and save new dwords 1 and 3 into xmm0
	pshufd	xmm0,xmm0,shuf(3,1,2,0);11011000b
	;now we have dwords 3 and 0 in place, so swap dwords 1 and 2 only to finalize positioning
	pshufd	xmm1,xmm1,shuf(3,1,2,0);11011000b
	retn

align 16
FrontXmm::
			xmm_rorHq	xmm2,xmm2
									xmm_xchgLq	xmm7,xmm7
			xmm_rorLq	xmm2,xmm2
									xmm_xchgLq	xmm6,xmm6
	xmm_swapdw	xmm3,xmm2
					xmm_hi2lo	xmm4,xmm1
			pand	xmm3,xmm6
					xmm_hi2hi	xmm4,xmm0
	pand	xmm2,xmm7
							xmm_mov		xmm5,xmm4
	xmm_rolLq	xmm0,xmm0
							pand	xmm5,xmm7
					pand	xmm4,xmm6
									xmm_xchgLq	xmm7,xmm7
	por		xmm2,xmm4
									xmm_xchgLq	xmm6,xmm6
							por		xmm5,xmm3
	xmm_rolLq	xmm2,xmm2
					xmm_lo2hi	xmm1,xmm5
	xmm_rolHq	xmm2,xmm2
					xmm_hi2hi	xmm0,xmm5
	retn

align 16
FrontXmmX::
			xmm_rolLq	xmm2,xmm2
									xmm_xchgHq	xmm7,xmm7
			xmm_rolHq	xmm2,xmm2
									xmm_xchgHq	xmm6,xmm6
	xmm_swapdw	xmm3,xmm2
					xmm_hi2lo	xmm4,xmm0
			pand	xmm3,xmm6
					xmm_hi2hi	xmm4,xmm1
	pand	xmm2,xmm7
							xmm_mov		xmm5,xmm4
	xmm_rorLq	xmm0,xmm0
							pand	xmm5,xmm7
					pand	xmm4,xmm6
									xmm_xchgHq	xmm7,xmm7
	por		xmm2,xmm4
									xmm_xchgHq	xmm6,xmm6
							por		xmm5,xmm3
	xmm_rorLq	xmm2,xmm2
					xmm_lo2hi	xmm0,xmm5
	xmm_rorHq	xmm2,xmm2
					xmm_hi2hi	xmm1,xmm5
	retn

align 16
BackXmm::
			xmm_rolLq	xmm2,xmm2
									xmm_xchgLq	xmm7,xmm7
			xmm_rolHq	xmm2,xmm2
									xmm_xchgLq	xmm6,xmm6
	xmm_swapdw	xmm3,xmm2
					xmm_hi2lo	xmm4,xmm0
	pand	xmm2,xmm7
					xmm_hi2hi	xmm4,xmm1
			pand	xmm3,xmm6
							xmm_mov		xmm5,xmm4
	xmm_rolLq	xmm1,xmm1
							pand	xmm5,xmm7
					pand	xmm4,xmm6
									xmm_xchgLq	xmm7,xmm7
	por		xmm2,xmm4
									xmm_xchgLq	xmm6,xmm6
							por		xmm5,xmm3
	xmm_rorLq	xmm2,xmm2
					xmm_lo2hi	xmm0,xmm5
	xmm_rorHq	xmm2,xmm2
					xmm_hi2hi	xmm1,xmm5
	retn

align 16
BackXmmX::
			xmm_rorHq	xmm2,xmm2
									xmm_xchgHq	xmm7,xmm7
			xmm_rorLq	xmm2,xmm2
									xmm_xchgHq	xmm6,xmm6
	xmm_swapdw	xmm3,xmm2
					xmm_hi2lo	xmm4,xmm1
	pand	xmm2,xmm7
					xmm_hi2hi	xmm4,xmm0
			pand	xmm3,xmm6
							xmm_mov		xmm5,xmm4
	xmm_rorLq	xmm1,xmm1
							pand	xmm5,xmm7
					pand	xmm4,xmm6
									xmm_xchgHq	xmm7,xmm7
	por		xmm2,xmm4
									xmm_xchgHq	xmm6,xmm6
							por		xmm5,xmm3
	xmm_rolLq	xmm2,xmm2
					xmm_lo2hi	xmm1,xmm5
	xmm_rolHq	xmm2,xmm2
					xmm_hi2hi	xmm0,xmm5
	retn


;assume esi:nothing	


;--------------------------------------------------------------------------------
